#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2018 KGH
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#

# -------------------------------------------------------------------------------------
# 
# This Version - Version 2.17
# Change Log
# Version 1.00 Joined-up writing Primer
# The idea of this plugin is to take some text and convert it into a series of unicode characters
# and display the text as joined up characters using a custom unicode font.
# Version 1.02 work on a group of similar Joiner fonts with letter variations.
# Version 1.03 adds letter variations
# There are a number of letter variation options (currently 8 of these) to tweak the font style output.
# Version 1.04 adds a new font - ItalicVariations
# The range of joins has been increased from 4 to 6: for letter e, for rounded letters like a, for slanted letters like v, for straight letters like i, for letter s, for letter z.
# Version 1.05 adds a new font weight - ItalicBoldVariations
# Smart quotes added: Kerning added to all fonts (although this is largely irrelevant in joined-up mode): Fixed letter s issue to force correct join with italic fonts and alternative letter s.
# Version 1.06 adds a new font weight - ItalicBlackVariations: Kerning added to non-joining lower-case letters.
#
# Version 2.00 Major update to place joined-up text on the given path and produces a non-text layer. Where there is no path available the filter will display the joined-up text as a text layer.
# Version 2.01 Fixed Retain Layers when set to yes bug
# Version 2.02 Added new font "SquareBeads" to list of fonts
# Version 2.03 Added unicode 'utf-8' encoding for text entry to display copyright sign, pound sign and division sign characters in fonts
# Version 2.04 Stopped filter from creating unnecessary layers at very small, or very large, sizes
# Version 2.05 Added text size modifiers option to increase and decrease the size of text along the path
# Version 2.06 Added shaded text option to change the colour of text along the path
# Version 2.07 Added coloured text option to change the colour of text along the path using the colours from the ui gradient tool
# Version 2.08 No longer continues to write characters beyond the length of the active path; kerning adjustments for more letter layers added; more glyphs added to fonts
# Version 2.09 Now works with ordinary fonts as well as joined-up fonts; replaced font selection dialog; added repeat text option to fill a path with text; removed compensation multiplier toggle and set compensation to 0 as default; altered some of the wording in dialog box options
# Version 2.10 Possibly fixed bug with occasional mis-rotation of characters; started to add smart label for final flattened text layer to indicate filter attributes
# Version 2.11 Occasional mis-rotation of characters seems to have been fixed now; Repeated text no longer leaves parts of repeats at the end of paths - padding space used to regulate positions;
#              repeated strings without spaces use padding between characters (apart from own joined-up fonts); removed multiple spaces from entry text with repeats; removed trailing spaces from entry text - to avoid sizing and padding issues
# Version 2.12 Changed unicode range for fonts to use private area pages. Therefore this version no longer works with the older fonts. New versions of fonts labelled appropriately. Not published.
# Version 2.13 Reversed text option added. This is a simplistic interpretation - left to right paths are copied, reversed and then added before flipping individual letters into the correct orientation along the active path. 
#              Version Number added to dialog title.
# Version 2.14 Altered layer label title format and reversed l-r path label title
# Version 2.15 New dialog options added: rotate letters; rotation jitter; layer opacity
# Version 2.16 Rotation jitter now available without having to first set angle of rotation
# Version 2.17 Added Custom XCF Font handling - all the effects that work with ordinary fonts will work with Custom XCF Fonts - undefined custom font layers caught, reported and displayed as spaces - multi-line text entry box replaced by single line box in filter dialog
#
# The plugin can be found via Filters>Text FX>Joined-up Text v2.15...
# If you have an older version of the plugin it is probabably best to remove it or rename it and relocate it
#
# The filter uses the function "computeThetaWithSlope" written by Ofnuts
# and parts of code taken from Tranductin's filter, custom-font_on_path, which he attributes to a post on http://stackoverflow.com/questions/31735499/calculate-angle-clockwise-between-two-points
# Without their work I would not have been able to create this filter

# Import Gimp Python modules

from array import array
from gtk import gdk
from gimpfu import *
import math	
import random


# Define some global variables

w = gdk.screen_width()
h = gdk.screen_height()

last_loop = 0
#copy_of_joined_up_text = ""

# Define some colours we may need to use

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
darkgrey = gimpcolor.RGB(50, 50, 50)
midgrey = gimpcolor.RGB(125, 125, 125)
lightgrey = gimpcolor.RGB(170, 170, 170)
verylightgrey = gimpcolor.RGB(222, 222, 222)

# Define strings containing joining letters

a_joins = unichr(57344)+unichr(57345)+unichr(57346)+unichr(57347)+unichr(57348)+unichr(57349)
c_joins = unichr(57356)+unichr(57357)+unichr(57358)+unichr(57359)+unichr(57360)+unichr(57361)
d_joins = unichr(57362)+unichr(57363)+unichr(57364)+unichr(57365)+unichr(57366)+unichr(57367)
e_joins = unichr(57368)+unichr(57369)+unichr(57370)+unichr(57371)+unichr(57372)+unichr(57373)+unichr(57374)+unichr(57375)+unichr(57376)+unichr(57377)+unichr(57378)+unichr(57379)+unichr(57380)+unichr(57381)
g_joins = unichr(57387)+unichr(57388)+unichr(57389)+unichr(57390)+unichr(57391)+unichr(57509)+unichr(57510)+unichr(57511)+unichr(57512)+unichr(57513)+unichr(57514)+unichr(57515)
o_joins = unichr(57433)+unichr(57434)+unichr(57435)+unichr(57436)+unichr(57437)
q_joins = unichr(57444)+unichr(57445)+unichr(57446)+unichr(57447)+unichr(57448)+unichr(57449)
lower_case_1 = "acdegoq"+a_joins+c_joins+d_joins+e_joins+g_joins+o_joins+q_joins
b_joins = unichr(57350)+unichr(57351)+unichr(57352)+unichr(57353)+unichr(57354)+unichr(57355)
h_joins = unichr(57392)+unichr(57393)+unichr(57394)+unichr(57395)+unichr(57396)+unichr(57397)
i_joins = unichr(57398)+unichr(57399)+unichr(57400)+unichr(57401)+unichr(57402)+unichr(57403)
k_joins = unichr(57409)+unichr(57410)+unichr(57411)+unichr(57412)+unichr(57413)+unichr(57414)+unichr(57523)+unichr(57524)+unichr(57525)+unichr(57526)+unichr(57527)+unichr(57528)+unichr(57529)
l_joins = unichr(57415)+unichr(57416)+unichr(57417)+unichr(57418)+unichr(57419)+unichr(57420)
lower_case_2 = "bhikl"+b_joins+h_joins+i_joins+k_joins+l_joins
m_joins = unichr(57421)+unichr(57422)+unichr(57423)+unichr(57424)+unichr(57425)+unichr(57426)
n_joins = unichr(57427)+unichr(57428)+unichr(57429)+unichr(57430)+unichr(57431)+unichr(57432)
r_joins = unichr(57450)+unichr(57451)+unichr(57452)+unichr(57453)+unichr(57454)
u_joins = unichr(57467)+unichr(57468)+unichr(57469)+unichr(57470)+unichr(57471)+unichr(57472)
y_joins = unichr(57488)+unichr(57489)+unichr(57490)+unichr(57491)+unichr(57492)+unichr(57493)+unichr(57551)+unichr(57552)+unichr(57553)+unichr(57554)+unichr(57555)+unichr(57556)+unichr(57557)
lower_case_3 = "mnruy"+m_joins+n_joins+r_joins+u_joins+y_joins
v_joins = unichr(57473)+unichr(57474)+unichr(57475)+unichr(57476)+unichr(57477)
w_joins = unichr(57478)+unichr(57479)+unichr(57480)+unichr(57481)+unichr(57482)
lower_case_4 = "vw"+v_joins+w_joins
x_joins = unichr(57483)+unichr(57484)+unichr(57485)+unichr(57486)+unichr(57487)
z_joins = unichr(57493)+unichr(57494)+unichr(57495)+unichr(57496)+unichr(57497)+unichr(57498)
lower_case_5 = "xz"+x_joins+z_joins
alt_x_joins = unichr(57544)+unichr(57545)+unichr(57546)+unichr(57547)+unichr(57548)+unichr(57549)+unichr(57550)
lower_case_6 = alt_x_joins
alt_z_joins = unichr(57558)+unichr(57559)+unichr(57560)+unichr(57561)+unichr(57562)+unichr(57563)+unichr(57564)
lower_case_7 = alt_z_joins
s_joins = unichr(57455)+unichr(57456)+unichr(57457)+unichr(57458)+unichr(57459)+unichr(57460)+unichr(57530)+unichr(57531)+unichr(57532)+unichr(57533)+unichr(57534)+unichr(57535)+unichr(57536)+unichr(57537)+unichr(57538)+unichr(57539)+unichr(57540)+unichr(57541)+unichr(57542)+unichr(57543)
lower_case_8 = "s"+s_joins

# horizontal adjustment information for layers created by characters with negative font positions ie fjp and t
f_alone_left_bearing = [208,216,216,280,0] # letter f standalone x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
f_alone_right_bearing = [-190,-204,-188,-236,0] # letter f standalone x position right bearing character adjustment for each joining font (italic, bold, black, joiner)
f_left_bearing = [208,216,216,280,0] # letter f  x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
f_right_bearing = [-190,-204,-188,-236,0] # letter f  x position right  bearing character adjustment for each joining font (italic, bold, black, joiner)

alt_f_alone_left_bearing = [48,64,48,80,0] # alternative letter f standalone x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
alt_f_alone_right_bearing = [-32,-32,-16,-48,0] # alternative letter f standalone x position right bearing character adjustment for each joining font (italic, bold, black, joiner)
alt_f_left_bearing = [48,64,48,80,0] # alternative letter f  x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
alt_f_right_bearing = [-20,-32,-16,-48,0] # alternative letter f  x position right  bearing character adjustment for each joining font (italic, bold, black, joiner)
	
j_alone_left_bearing = [220,232,226,284,0] # letter j standalone x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
j_alone_right_bearing = [-220,-232,-240,-280,0] # letter j standalone x position right bearing character adjustment for each joining font (italic, bold, black, joiner)
j_left_bearing = [220,232,226,284,0] # letter j x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
j_right_bearing = [-220,-232,-240,-280,0] # letter j x position right  bearing character adjustment for each joining font (italic, bold, black, joiner)

alt_j_alone_left_bearing = [248,276,248,284,0] # letter j standalone x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
alt_j_alone_right_bearing = [-240,-256,-256,-216,0] # letter j standalone x position right bearing character adjustment for each joining font (italic, bold, black, joiner)
alt_j_left_bearing = [248,276,248,284,0] # letter j x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
alt_j_right_bearing = [-240,-256,-256,-216,0] # letter j x position right  bearing character adjustment for each joining font (italic, bold, black, joiner)
	
p_alone_left_bearing = [160,176,172,0,0] # letter p standalone x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
p_alone_right_bearing = [-152,-152,-148,0,0] # letter p standalone x position right bearing character adjustment for each joining font (italic, bold, black, joiner)
p_left_bearing = [160,176,172,0,0] # letter p x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
p_right_bearing = [-152,-152,-148,0,0] # letter p x position right  bearing character adjustment for each joining font (italic, bold, black, joiner)
	
t_alone_left_bearing = [44,52,48,60,0] # letter t standalone x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
t_alone_right_bearing = [-28,-36,-24,-28,0] # letter t standalone x position right bearing character adjustment for each joining font (italic, bold, black, joiner)
t_left_bearing = [44,52,48,60,0] # letter t x position left bearing character adjustment for each joining font (italic, bold, black, joiner)
t_right_bearing = [-28,-36,-24,-28,0] # letter t x position right  bearing character adjustment for each joining font (italic, bold, black, joiner)

# horizontal adjustment information for layers following those created by Capital letters 

A_to_ABDEFHKLMNPRS_right_bearing = [0,0,0,60,0]
A_to_J_right_bearing = [203,203,203,80,0]
A_to_CGOQ_right_bearing = [0,0,0,100,0]
A_to_T_right_bearing = [203,203,203,180,0]
A_to_VW_right_bearing = [104,159,145,200,0]
A_to_IU_right_bearing = [0,0,0,50,0]
A_to_Y_right_bearing = [67,94,67,180,0]
A_to_Z_right_bearing = [67,94,67,60,0]
A_to_lc1_right_bearing = [58,31,17,60,0]
A_to_lc2_3_right_bearing = [45,25,14,0,0]
A_to_lc4_right_bearing = [113,113,86,118,0]
A_to_lc5_right_bearing = [45,94,14,0,0]
A_to_lc6_right_bearing = [81,90,69,48,0]
A_to_lc7_right_bearing = [145,130,110,145,0]
A_to_lc8_right_bearing = [83,60,55,109,0]

B_to_A_right_bearing = [0,0,42,80,0]
B_to_BDEFLRS_right_bearing = [0,0,62,27,0]
B_to_CGOQ_right_bearing = [0,0,0,60,0]
B_to_I_right_bearing = [0,0,0,80,0]
B_to_T_right_bearing = [0,0,0,120,0]
B_to_VW_right_bearing = [0,0,73,100,0]
B_to_XYZ_right_bearing = [0,0,51,100,0]
B_to_lc4_right_bearing = [0,0,0,80,0]
B_to_lc5_right_bearing = [0,0,42,50,0]
B_to_lc6_lc7_right_bearing = [70,0,56,0,0]
B_to_lc8_right_bearing = [56,0,56,50,0]

C_to_A_right_bearing = [52,107,134,100,0]
C_to_BDEFLR_right_bearing = [0,63,77,50,0]
C_to_IKMNPU_right_bearing = [0,0,0,50,0]
C_to_CGOQS_right_bearing = [0,0,73,60,0]
C_to_M_right_bearing = [0,78,105,0,0]
C_to_JT_right_bearing = [0,85,126,80,0]
C_to_VW_right_bearing = [0,0,0,100,0]
C_to_XYZ_right_bearing = [54,54,95,100,0]
C_to_lc1_right_bearing = [55,100,82,0,0]
C_to_lc2_right_bearing = [0,50,55,0,0]
C_to_lc3_right_bearing = [0,50,82,0,0]
C_to_lc4_right_bearing = [51,55,82,0,0]
C_to_lc5_right_bearing = [55,55,82,30,0]
C_to_lc6_right_bearing = [75,82,123,0,0]
C_to_lc7_right_bearing = [100,123,137,0,0]
C_to_lc8_right_bearing = [75,82,109,60,0]

D_to_A_right_bearing = [82,82,109,100,0]
D_to_BDEFLR_right_bearing = [0,99,72,50,0]
D_to_HKNPUCGOQ_right_bearing = [0,0,0,50,0]
D_to_I_right_bearing = [0,0,0,60,0]
D_to_J_right_bearing = [219,137,123,60,0]
D_to_M_right_bearing = [0,86,72,50,0]
D_to_S_right_bearing = [85,85,85,50,0]
D_to_T_right_bearing = [66,66,121,150,0]
D_to_VW_right_bearing = [0,55,55,0,0]
D_to_XYZ_right_bearing = [109,109,136,100,0]
D_to_lc1_right_bearing = [55,0,41,0,0]
D_to_lc8_right_bearing = [78,41,54,0,0]

E_to_A_right_bearing = [0,55,28,50,0]
E_to_VWMXY_right_bearing = [0,0,0,50,0]
E_to_CGOQS_right_bearing = [0,0,62,80,0]
E_to_JT_right_bearing = [86,86,110,50,0]
E_to_lc1_right_bearing = [0,0,0,40,0]
E_to_lc4_right_bearing = [96,82,69,40,0]
E_to_lc6_right_bearing = [55,27,55,0,0]
E_to_lc7_right_bearing = [110,82,124,0,0]
E_to_lc8_right_bearing = [55,27,28,30,0]

F_to_A_right_bearing = [187,187,255,150,0]
F_to_J_right_bearing = [200,200,200,200,0]
F_to_M_right_bearing = [94,94,121,40,0]
F_to_BDEHLNU_right_bearing = [98,98,100,40,0]
F_to_CGOQ_right_bearing = [98,98,125,90,0]
F_to_FIKPRVW_right_bearing = [50,50,90,40,0]
F_to_S_right_bearing = [140,140,140,90,0]
F_to_T_right_bearing = [73,73,100,40,0]
F_to_XYZ_right_bearing = [85,85,100,60,0]
F_to_lc1_right_bearing = [186,186,200,80,0]
F_to_lc2_right_bearing = [40,40,63,0,0]
F_to_lc3_right_bearing = [137,110,137,50,0]
F_to_lc4_right_bearing = [117,117,117,40,0]
F_to_lc5_right_bearing = [125,100,125,40,0]
F_to_lc6_right_bearing = [125,83,125,40,0]
F_to_lc7_right_bearing = [109,83,137,40,0]
F_to_lc8_right_bearing = [193,186,213,60,0]

G_to_A_right_bearing = [0,59,155,100,0]
G_to_M_right_bearing = [0,59,155,40,0]
G_to_BDEFHIKLMNPRUS_right_bearing = [0,0,0,40,0]
G_to_J_right_bearing = [140,117,170,100,0]
G_to_T_right_bearing = [76,117,158,100,0]
G_to_XYZ_right_bearing = [0,67,136,100,0]
G_to_VW_right_bearing = [0,0,0,100,0]
G_to_lc1_right_bearing = [40,90,99,50,0]
G_to_lc2_lc3_right_bearing = [0,0,41,0,0]
G_to_lc4_lc5_right_bearing = [0,0,25,0,0]
G_to_lc6_right_bearing = [0,-53,43,0,0]
G_to_lc7_right_bearing = [0,-66,43,0,0]
G_to_lc8_right_bearing = [0,90,98,50,0]

H_to_A_right_bearing = [52,52,52,40,0]
H_to_J_right_bearing = [100,100,100,20,0]
H_to_M_right_bearing = [0,93,38,20,0]
H_to_T_right_bearing = [50,47,20,0,0]
H_to_VWXYZ_right_bearing = [0,0,0,40,0]
H_to_BDEFHIKLNPRU_right_bearing = [0,0,0,20,0]
H_to_COGQ_right_bearing = [0,0,0,20,0]
H_to_lc1_right_bearing = [46,41,13,0,0]
H_to_lc2_lc3_right_bearing = [41,41,13,0,0]
H_to_lc4_lc5_right_bearing = [41,41,13,0,0]
H_to_lc6_lc7_right_bearing = [0,38,13,0,0]
H_to_lc8_right_bearing = [67,65,40,40,0]

I_to_AM_right_bearing = [0,0,57,40,0]
I_to_B_right_bearing = [60,40,44,40,0]
I_to_CGOQ_right_bearing = [0,0,0,60,0]
I_to_DEFHIKLNPRUS_right_bearing = [0,0,0,40,0]
I_to_J_right_bearing = [104,104,104,60,0]
I_to_T_right_bearing = [60,60,87,40,0]
I_to_VW_right_bearing = [0,0,69,40,0]
I_to_XYZ_right_bearing = [0,0,0,40,0]
I_to_lc1_right_bearing = [40,40,40,50,0]
I_to_lc4_right_bearing = [40,40,55,40,0]
I_to_lc6_right_bearing = [0,28,42,0,0]
I_to_lc7_right_bearing = [0,41,77,0,0]
I_to_lc8_right_bearing = [0,14,42,50,0]

J_to_A_right_bearing = [0,0,0,120,0]
J_to_BDEFHIKLMNPRTUXYZ_right_bearing = [0,0,0,40,0]
J_to_CGOQS_right_bearing = [0,0,0,100,0]
J_to_VW_right_bearing = [0,0,0,50,0]
J_to_lc1_right_bearing = [47,47,27,80,0]
J_to_lc2_right_bearing = [40,22,12,0,0]
J_to_lc3_right_bearing = [40,26,16,0,0]
J_to_lc4_lc5_right_bearing = [40,41,11,50,0]
J_to_lc6_lc7_right_bearing = [49,41,21,50,0]
J_to_lc8_right_bearing = [88,47,47,80,0]

K_to_AUVWXY_right_bearing = [40,40,40,60,0]
K_to_BDEFHIKLMNPRZ_right_bearing = [0,0,40,40,0]
K_to_CGOQ_right_bearing = [60,60,60,120,0]
K_to_J_right_bearing = [152,152,152,100,0]
K_to_S_right_bearing = [0,0,60,100,0]
K_to_T_right_bearing = [80,121,80,40,0]
K_to_lc1_right_bearing = [45,18,13,80,0]
K_to_lc4_right_bearing = [86,86,59,100,0]
K_to_lc5_right_bearing = [45,45,27,0,0]
K_to_lc6_right_bearing = [0,98,27,0,0]
K_to_lc7_right_bearing = [100,139,95,0,0]
K_to_lc8_right_bearing = [66,59,54,100,0]

L_to_AX_right_bearing = [0,0,0,40,0]
L_to_BEFR_right_bearing = [0,0,60,40,0]
L_to_DHIKLMNPSUZ_right_bearing = [0,0,0,40,0]
L_to_CGOQ_right_bearing = [0,0,60,120,0]
L_to_J_right_bearing = [259,259,203,60,0]
L_to_T_right_bearing = [207,207,203,200,0]
L_to_VW_right_bearing = [0,0,140,140,0]
L_to_Y_right_bearing = [0,0,0,160,0]
L_to_lc1_right_bearing = [58,27,41,50,0]
L_to_lc2_lc3_right_bearing = [40,40,41,0,0]
L_to_lc4_right_bearing = [93,68,93,100,0]
L_to_lc6_right_bearing = [74,54,52,0,0]
L_to_lc7_right_bearing = [156,122,107,0,0]
L_to_lc8_right_bearing = [57,54,52,40,0]

M_to_AVWXY_right_bearing = [0,0,0,40,0]
M_to_BDEFHIKLMNPRSUZ_right_bearing = [0,0,0,40,0]
M_to_CGOQ_right_bearing = [0,0,0,50,0]
M_to_J_right_bearing = [128,108,128,40,0]
M_to_T_right_bearing = [92,92,92,20,0]
M_to_lc1_right_bearing = [55,17,27,0,0]
M_to_lc2_lc3_right_bearing = [43,27,27,0,0]
M_to_lc4_right_bearing = [66,54,27,0,0]
M_to_lc6_lc7_right_bearing = [66,40,60,0,0]
M_to_lc8_right_bearing = [66,40,27,0,0]

N_to_AVWXY_right_bearing = [0,0,0,40,0]
N_to_BDEFHIKLMNPRTUZ_right_bearing = [0,0,0,40,0]
N_to_CGOQ_right_bearing = [0,0,0,50,0]
N_to_J_right_bearing = [100,80,80,40,0]
N_to_lc1_right_bearing = [41,0,14,0,0]
N_to_lc2_lc3_right_bearing = [41,0,14,0,0]
N_to_lc6_lc7_right_bearing = [41,0,14,0,0]
N_to_lc8_right_bearing = [61,0,41,40,0]

O_to_A_right_bearing = [112,153,112,120,0]
O_to_BDEFHKLNPRU_right_bearing = [0,0,0,40,0]
O_to_I_right_bearing = [0,0,0,60,0]
O_to_J_right_bearing = [112,112,112,50,0]
O_to_M_right_bearing = [64,105,64,40,0]
O_to_VWXYZ_right_bearing = [0,0,0,100,0]
O_to_lc1_right_bearing = [91,91,77,0,0]
O_to_lc2_lc3_right_bearing = [50,40,9,0,0]
O_to_lc4_right_bearing = [41,0,10,0,0]
O_to_lc7_right_bearing = [51,0,10,0,0]
O_to_lc8_right_bearing = [105,91,77,0,0]

P_to_A_right_bearing = [100,100,100,100,0]
P_to_BDEFHKLNPRU_right_bearing = [40,0,0,40,0]
P_to_CGOQ_right_bearing = [40,40,40,70,0]
P_to_I_right_bearing = [0,0,0,70,0]
P_to_J_right_bearing = [200,112,100,200,0]
P_to_M_right_bearing = [120,126,77,40,0]
P_to_S_right_bearing = [50,50,50,0,0]
P_to_T_right_bearing = [0,0,0,90,0]
P_to_VW_right_bearing = [0,0,0,70,0]
P_to_XYZ_right_bearing = [60,60,60,90,0]
P_to_lc1_right_bearing = [135,135,135,100,0]
P_to_lc2_right_bearing = [45,45,0,0,0]
P_to_lc3_right_bearing = [45,45,45,0,0]
P_to_lc4_right_bearing = [60,74,46,0,0]
P_to_lc5_right_bearing = [67,67,67,0,0]
P_to_lc6_right_bearing = [53,53,58,0,0]
P_to_lc7_right_bearing = [80,80,58,0,0]
P_to_lc8_right_bearing = [130,130,135,100,0]

Q_to_AJS_right_bearing = [0,0,0,60,0]
Q_to_BDEFHKLMNPRUZ_right_bearing = [0,0,0,60,0]
Q_to_CGOQ_right_bearing = [0,0,0,60,0]
Q_to_T_right_bearing = [0,0,0,90,0]
Q_to_VW_right_bearing = [0,0,0,100,0]
Q_to_XY_right_bearing = [0,0,0,90,0]
Q_to_lc1_right_bearing = [40,40,26,40,0]
Q_to_lc6_right_bearing = [46,26,0,0,0]
Q_to_lc7_right_bearing = [46,26,0,0,0]
Q_to_lc8_right_bearing = [65,40,26,40,0]

R_to_ABDEFHIKLMNRPXZ_right_bearing = [40,40,40,40,0]
R_to_CGOQ_right_bearing = [40,40,40,110,0]
R_to_J_right_bearing = [150,173,146,110,0]
R_to_SU_right_bearing = [0,0,0,60,0]
R_to_T_right_bearing = [110,104,104,100,0]
R_to_VWY_right_bearing = [40,40,40,120,0]
R_to_lc1_right_bearing = [40,40,40,80,0]
R_to_lc4_right_bearing = [90,104,90,80,0]
R_to_lc6_right_bearing = [54,76,63,0,0]
R_to_lc7_right_bearing = [122,117,107,0,0]
R_to_lc8_right_bearing = [40,40,26,80,0]

S_to_A_right_bearing = [101,128,101,100,0]
S_to_BDEFHIKLNPRU_right_bearing = [40,40,40,40,0]
S_to_CGOQ_right_bearing = [40,40,40,60,0]
S_to_I_right_bearing = [0,0,0,20,0]
S_to_J_right_bearing = [150,124,104,60,0]
S_to_M_right_bearing = [92,147,92,40,0]
S_to_S_right_bearing = [60,60,60,0,0]
S_to_T_right_bearing = [64,64,50,60,0]
S_to_VW_right_bearing = [0,0,0,100,0]
S_to_XY_right_bearing = [108,108,81,100,0]
S_to_Z_right_bearing = [108,108,81,40,0]
S_to_lc1_right_bearing = [84,97,70,0,0]
S_to_lc2_right_bearing = [40,40,40,0,0]
S_to_lc3_right_bearing = [69,83,42,0,0]
S_to_lc4_right_bearing = [94,108,67,40,0]
S_to_lc5_right_bearing = [100,100,100,40,0]
S_to_lc6_lc7_right_bearing = [127,160,100,40,0]
S_to_lc8_right_bearing = [133,97,90,40,0]

T_to_A_right_bearing = [76,160,213,160,0]
T_to_J_right_bearing = [60,160,154,160,0]
T_to_BDEFLR_right_bearing = [-40,40,97,40,0]
T_to_HKNPU_right_bearing = [0,0,0,40,0]
T_to_CGOQ_right_bearing = [0,80,160,140,0]
T_to_I_right_bearing = [-70,-50,40,0,0]
T_to_M_right_bearing = [0,35,145,40,0]
T_to_S_right_bearing = [0,100,160,80,0]
T_to_VW_right_bearing = [0,40,0,60,0]
T_to_XYZ_right_bearing = [-40,40,100,60,0]
T_to_lc1_right_bearing = [110,200,200,180,0]
T_to_lc2_right_bearing = [-40,100,60,0,0]
T_to_lc3_right_bearing = [95,95,170,160,0]
T_to_lc4_right_bearing = [124,124,190,160,0]
T_to_lc5_right_bearing = [95,109,200,160,0]
T_to_lc6_lc7_right_bearing = [96,100,200,160,0]
T_to_lc8_right_bearing = [138,138,200,180,0]

U_to_A_right_bearing = [80,85,80,80,0]
U_to_B_right_bearing = [99,126,58,40,0]
U_to_DEL_right_bearing = [86,86,60,40,0]
U_to_F_right_bearing = [47,47,47,40,0]
U_to_HIKNPSUCGOQ_right_bearing = [0,0,0,40,0]
U_to_R_right_bearing = [68,68,66,40,0]
U_to_XYZ_right_bearing = [68,68,66,60,0]
U_to_M_right_bearing = [80,85,80,40,0]
U_to_VW_right_bearing = [80,85,80,60,0]
U_to_J_right_bearing = [172,172,172,40,0]
U_to_T_right_bearing = [126,153,126,40,0]
U_to_lc1_right_bearing = [46,46,20,0,0]
U_to_lc2_lc3_right_bearing = [49,41,0,0,0]
U_to_lc4_lc5_right_bearing = [83,70,60,0,0]
U_to_lc6_lc7_right_bearing = [124,96,83,0,0]
U_to_lc8_right_bearing = [42,69,73,0,0]

V_to_A_right_bearing = [154,154,154,200,0]
V_to_BDEFHIKLNPRSTUVWXYZ_right_bearing = [0,0,0,40,0]
V_to_CGOQ_right_bearing = [0,0,0,80,0]
V_to_J_right_bearing = [88,74,88,160,0]
V_to_M_right_bearing = [74,88,88,40,0]
V_to_lc1_right_bearing = [85,100,123,100,0]
V_to_lc3_right_bearing = [0,0,0,100,0]
V_to_lc4_lc5_right_bearing = [0,0,0,100,0]
V_to_lc6_lc7_right_bearing = [0,0,0,100,0]
V_to_lc8_right_bearing = [82,100,123,100,0]

W_to_A_right_bearing = [141,141,141,200,0]
W_to_BDEFHIKLNPRSTUVWXYZ_right_bearing = [0,0,0,40,0]
W_to_CGOQ_right_bearing = [0,0,0,80,0]
W_to_J_right_bearing = [88,88,88,160,0]
W_to_M_right_bearing = [63,110,83,40,0]
W_to_lc1_right_bearing = [105,78,105,100,0]
W_to_lc3_right_bearing = [0,0,0,100,0]
W_to_lc4_right_bearing = [0,0,0,100,0]
W_to_lc5_right_bearing = [45,4,18,100,0]
W_to_lc6_lc7_right_bearing = [45,17,20,100,0]
W_to_lc8_right_bearing = [119,78,105,100,0]

X_to_A_right_bearing = [68,68,68,40,0]
X_to_BDEFHIKLNPRSTUVWXYZ_right_bearing = [0,0,0,40,0]
X_to_CGOQ_right_bearing = [0,0,0,100,0]
X_to_J_right_bearing = [100,100,100,100,0]
X_to_M_right_bearing = [68,68,68,40,0]
X_to_lc1_right_bearing = [40,40,13,60,0]
X_to_lc4_right_bearing = [60,70,60,100,0]
X_to_lc5_right_bearing = [60,70,60,20,0]
X_to_lc7_right_bearing = [170,150,87,0,0]
X_to_lc8_right_bearing = [40,40,40,80,0]

Y_to_A_right_bearing = [182,200,237,200,0]
Y_to_BDEFHIKLNPRTUVWXYZ_right_bearing = [40,60,60,40,0]
Y_to_CGOQ_right_bearing = [80,120,100,100,0]
Y_to_J_right_bearing = [150,200,107,160,0]
Y_to_M_right_bearing = [59,100,141,40,0]
Y_to_S_right_bearing = [100,100,100,40,0]
Y_to_T_right_bearing = [59,40,60,40,0]
Y_to_lc1_lc8_right_bearing = [223,220,278,140,0]
Y_to_lc2_right_bearing = [0,60,57,0,0]
Y_to_lc3_right_bearing = [120,120,175,100,0]
Y_to_lc4_right_bearing = [160,133,187,100,0]
Y_to_lc5_right_bearing = [140,144,194,100,0]
Y_to_lc6_lc7_right_bearing = [140,126,180,100,0]

Z_to_A_right_bearing = [72,100,80,60,0]
Z_to_BDEFHIKLNPRUVWXYZ_right_bearing = [0,0,0,40,0]
Z_to_J_right_bearing = [72,100,120,120,0]
Z_to_M_right_bearing = [72,100,80,40,0]
Z_to_CGOQS_right_bearing = [100,100,63,80,0]
Z_to_T_right_bearing = [89,89,89,40,0]
Z_to_lc1_right_bearing = [44,44,44,40,0]
Z_to_lc2_lc3_right_bearing = [58,31,44,0,0]
Z_to_lc4_right_bearing = [80,135,94,60,0]
Z_to_lc5_right_bearing = [44,44,44,0,0]
Z_to_lc6_right_bearing = [114,86,44,60,0]
Z_to_lc7_right_bearing = [155,113,126,60,0]
Z_to_lc8_right_bearing = [90,72,72,40,0]


my_debug = 0
#######################################################################################################

# Define Functions/Procedures

####################################################################################################### 

def proc_main_plugin(image, layer, ui_font_name, font_size, font_colour, blend_colour, spread_colour, joinedup_gradient, use_gradient_colour, xcffilename, use_custom,
                    text, tranparency, reversed_writing, do_repeat, step_size, use_text_size_modifier, initial_text_size, medial_text_size, final_text_size, 
                    position_of_medial, use_space_to_change_step_size, compensate_multiplier,
                    kern_amount, rotate_text, user_angle, jitter, jitter_range, keep_layers, keep_paths, f_nohook_flag, g_loop_flag, 
                    j_loop_flag, k_unloop_flag, s_closed_flag, x_rounded_flag, y_loop_flag, 
                    z_loop_flag):
    
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)
	
    #Define variables for font-joining routine
    force_eflag = True
    remaining_path = 0
    # Remove any newline characters	and leading spaces
    text = fn_clean_up_text(text)
	
    #Rem: force_eflag determines what type of letter e should follow an unjoined hooked letter
    #Rem: Looped hook fonts do not require the flag to be set
    force_sflag = False
    #Rem: force_sflag determines whether letter s changes to accomodate joins from looped and unlooped e
    #Rem: Italic fonts do not shape e character in different ways, therefore force_sflag must be set to True
	
    #v 2.17 line
    fontimg = -1
    if xcffilename == "": use_custom = False	
	
    # Get font name and set up joined-up text font pointer
	
    if (ui_font_name == "ItalicVariations Italic"):
        font_name_pointer = 0
        force_sflag = True
    elif (ui_font_name == "ItalicBoldVariations Italic"):
        font_name_pointer = 1
        force_sflag = True
    elif (ui_font_name == "ItalicBlackVariations Italic"):
        font_name_pointer = 2
        force_sflag = True
    elif (ui_font_name == "JoinerVariations"):
        font_name_pointer = 3
    elif (ui_font_name == "SquareBeads"):
        font_name_pointer = 4
    else:
        font_name_pointer = -1
    
    font_name = ui_font_name
	
    if use_custom:
        missing_layers = False
        font_name_pointer, file_dummy = -1, 0
        fontimg = pdb.gimp_xcf_load(file_dummy,xcffilename,xcffilename)
        text, missing_layers, missing_amount = fn_check_xcf_text(text,fontimg, missing_layers)
    
    text = text.decode('utf-8') # Allows unicode characters above 127 to be displayed without creating errors
    
    # Get path/vector information
    # num_vectors holds the number of paths associated with the current image
    # vector_ids holds the list of ids for the paths associated with the current image
    num_vectors, vector_ids = pdb.gimp_image_get_vectors(image)
    pdb.gimp_selection_none(image)

    if (num_vectors == 0):
        # No path present so report to user
        old_handler = pdb.gimp_message_get_handler()		
        pdb.gimp_message_set_handler(MESSAGE_BOX)
        pdb.gimp_message("There is no path to work with: \nShowing joined-up text as a single line instead.")	
        pdb.gimp_message_set_handler(old_handler)
        if font_name_pointer > -1:
            # Convert text to joined-up text
            new_text, name_layer = fn_convert_text_to_joined_up_text(text, force_eflag, force_sflag, f_nohook_flag, g_loop_flag, j_loop_flag, k_unloop_flag, s_closed_flag, x_rounded_flag, y_loop_flag, z_loop_flag)
            # Draw the joined-up text 
            proc_show_joined_up_text_line(image, layer, new_text, font_name, font_colour, font_size, 0, 100, name_layer)
        else:
            proc_show_joined_up_text_line(image, layer, text, font_name, font_colour, font_size, 0, 100, text)
    else:
		
        if use_custom and missing_layers: # Report missing characters
            old_handler = pdb.gimp_message_get_handler()		
            pdb.gimp_message_set_handler(MESSAGE_BOX)
            if missing_amount == 1:
                pdb.gimp_message("One non-letter character is missing from the XCF-Font file and has been replaced with a space.\nYou may wish to edit your text in light of this.")
            else:
                pdb.gimp_message(str(missing_amount) + " non-letter characters are missing from the XCF-Font file and have been replaced with spaces.\nYou may wish to edit your text in light of this.")	
            pdb.gimp_message_set_handler(old_handler)
		
        user_angle = fn_angle_value(user_angle) #converts a negative user angle to a positive value
        Left_to_Right_path = False
		
        if reversed_writing:
            # Check to see if user has asked for reversed writing along a L - R path
            # See if there is an active path and if not set the first available path to be active
            active_vector = pdb.gimp_image_get_active_vectors(image)
            if (active_vector == -1):
                active_vector = pdb.gimp_image_set_active_vectors(image,image.vectors[0])
	
            active_vectors = pdb.gimp_image_get_active_vectors(image)
		
            # Get path stroke information
            NumberOfStrokes_in_Path,ActivePath_StrokeIDs_List = pdb.gimp_vectors_get_strokes(active_vector)
            ActivePath_StrokeID = ActivePath_StrokeIDs_List[0]
            path_length = pdb.gimp_vectors_stroke_get_length(active_vectors,ActivePath_StrokeID,0.01)
            Type_of_Stroke, Number_of_Points, Control_Point_List, Stroke_Closed = pdb.gimp_vectors_stroke_get_points(active_vector, ActivePath_StrokeID)
            First_Point_x_on_Active_Path = float(Control_Point_List[2])
            Last_Point_x_on_Active_Path = float(Control_Point_List[Number_of_Points - 4])
            #pdb.gimp_message("Control_Point_List = " + str(Control_Point_List) )
            Left_to_Right_path = False
            if First_Point_x_on_Active_Path < Last_Point_x_on_Active_Path: 
                Left_to_Right_path = True
                Reversed_Control_Point_List = fn_Reverse_Control_Point_List(Control_Point_List, Number_of_Points)
                #pdb.gimp_message("Reversed_Control_Point_List = " + str(Reversed_Control_Point_List) )
                new_vector = pdb.gimp_vectors_new(image,"Reversed Path")
                pdb.gimp_image_insert_vectors(image,new_vector,None,-1)
                stroke_id = pdb.gimp_vectors_stroke_new_from_points(new_vector,0,Number_of_Points,Reversed_Control_Point_List,False)
        if font_name_pointer > -1:
            # For own joined-up fonts convert text to joined-up text
            new_text, name_layer = fn_convert_text_to_joined_up_text(text, force_eflag, force_sflag, f_nohook_flag, g_loop_flag, j_loop_flag, k_unloop_flag, s_closed_flag, x_rounded_flag, y_loop_flag, z_loop_flag)
			
            if do_repeat:
                temp_text, text, temp_checkloop, temp_width, temp_path_len, remaining_path = fn_check_text_along_path(image, layer, new_text, name_layer, font_name_pointer,
                                    font_name, font_colour, blend_colour, spread_colour, joinedup_gradient, use_gradient_colour, font_size, step_size, kern_amount, 
                                    keep_layers, keep_paths, use_space_to_change_step_size, 
                                    compensate_multiplier, use_text_size_modifier, initial_text_size, medial_text_size, final_text_size, 
                                    position_of_medial, do_repeat, reversed_writing, Left_to_Right_path, rotate_text, user_angle, jitter, jitter_range, use_custom, xcffilename, fontimg)
	
                final_temp_text = temp_text
							
                proc_text_along_path(image, layer, final_temp_text, text, font_name_pointer,
                                    font_name, font_colour, blend_colour, spread_colour, joinedup_gradient, use_gradient_colour, font_size, step_size, kern_amount, 
                                    keep_layers, keep_paths, use_space_to_change_step_size, 
                                    compensate_multiplier, use_text_size_modifier, initial_text_size, medial_text_size, final_text_size, 
                                    position_of_medial, final_temp_text, do_repeat, remaining_path, reversed_writing, Left_to_Right_path, rotate_text, user_angle, jitter, jitter_range, tranparency, use_custom, xcffilename, fontimg)
									
            else:
                
                final_temp_text = new_text
                final_temp_text = final_temp_text.rstrip()
                text = text.rstrip()
                proc_text_along_path(image, layer, final_temp_text, text, font_name_pointer,
                                    font_name, font_colour, blend_colour, spread_colour, joinedup_gradient, use_gradient_colour, font_size, step_size, kern_amount, 
                                    keep_layers, keep_paths, use_space_to_change_step_size, 
                                    compensate_multiplier, use_text_size_modifier, initial_text_size, medial_text_size, final_text_size, 
                                    position_of_medial, final_temp_text, do_repeat, remaining_path, reversed_writing, Left_to_Right_path, rotate_text, user_angle, jitter, jitter_range, tranparency, use_custom, xcffilename, fontimg)
									
        else:
            # For not own joined-up fonts
					
            if do_repeat:	
									
                temp_text, text, temp_checkloop, temp_width, temp_path_len, remaining_path = fn_check_text_along_path(image, layer, text, text, font_name_pointer, 
                                    font_name, font_colour, blend_colour, spread_colour, joinedup_gradient, use_gradient_colour, font_size, step_size, kern_amount, 
                                    keep_layers, keep_paths, use_space_to_change_step_size, 
                                    compensate_multiplier, use_text_size_modifier, initial_text_size, medial_text_size, final_text_size, 
                                    position_of_medial, do_repeat, reversed_writing, Left_to_Right_path, rotate_text, user_angle, jitter, jitter_range, use_custom, xcffilename, fontimg)

                new_text = temp_text
                first_text = temp_text
			
                final_temp_text = temp_text
                compensate_multiplier = 0 # Messes up repeated text spacing with lots of repeats so not worth it for un-joined fonts
                proc_text_along_path(image, layer, final_temp_text, text, font_name_pointer,
                                font_name, font_colour, blend_colour, spread_colour, joinedup_gradient, use_gradient_colour, font_size, step_size, kern_amount, 
                                keep_layers, keep_paths, use_space_to_change_step_size, 
                                compensate_multiplier, use_text_size_modifier, initial_text_size, medial_text_size, final_text_size, 
                                position_of_medial, final_temp_text, do_repeat, remaining_path, reversed_writing, Left_to_Right_path, rotate_text, user_angle, jitter, jitter_range, tranparency, use_custom, xcffilename, fontimg)
								
            else:
                text = text.rstrip()
                final_temp_text = text
                proc_text_along_path(image, layer, final_temp_text, text, font_name_pointer,
                                    font_name, font_colour, blend_colour, spread_colour, joinedup_gradient, use_gradient_colour, font_size, step_size, kern_amount, 
                                    keep_layers, keep_paths, use_space_to_change_step_size, 
                                    compensate_multiplier, use_text_size_modifier, initial_text_size, medial_text_size, final_text_size, 
                                    position_of_medial, final_temp_text, do_repeat, remaining_path, reversed_writing, Left_to_Right_path, rotate_text, user_angle, jitter, jitter_range, tranparency, use_custom, xcffilename, fontimg)	
    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()

#######################################################################################################

def fn_check_xcf_text(text_to_check,fontimg, missing_layers):
    # converts any missing xcf-font characters to a space character
    new_text, loop, missing_amount, file_dummy = "",0,0,0
    while loop<len(text_to_check):
        char = text_to_check[loop]
        if (char >= "A" and char <= "Z") or (char >= "a" and char <= "z") or char == " ":
            new_text += char
        else:
            layer_name = char + ".png"
            char_layer = pdb.gimp_image_get_layer_by_name(fontimg,layer_name)
            if char_layer >= 0:
                new_text += char
            else:
                new_text += " "
                missing_layers = True
                missing_amount += 1
        loop += 1
    return new_text, missing_layers, missing_amount

#######################################################################################################

def fn_get_custom_font_scale(f_size, custom_font_height):
    new_custom_height = f_size * 1.164
    scale = new_custom_height / custom_font_height
    return scale

#######################################################################################################


def fn_angle_jitter(angle, jitter_range):
    half_of_jitter = int(jitter_range / 2)
    random.seed()
    jitter = 0
    start_angle = angle
    if jitter_range > 1:
        random_amount = random.randrange(1,jitter_range + 1,1)
        jitter = half_of_jitter - random_amount
    else:
        random_amount = random.randrange(1,3,1) - 1
        jitter = 1 + random_amount
    angle += jitter
    if angle < 0: angle += 359
    if angle > 359: angle = 359
    return angle
        
#######################################################################################################

def fn_angle_value(angle):
    if angle<0:
        if angle == -180:
            angle = 180
        else:
            angle += 360
    return angle

#######################################################################################################

def fn_Reverse_Control_Point_List(list, len):
    loop = len
    base = 0
    while loop > 0:
        if base == 0:
            new_list = [list[loop-2]]
            new_list.append(list[loop-1])
        else:
            new_list.append(list[loop-2])
            new_list.append(list[loop-1])
	
        base += 2
        loop -= 2
    return new_list
        
#######################################################################################################

def fn_get_gradient_positions(instances):
    # get positions along gradients for text colours
    grad_position = 0.0
    if instances<1: instances = 1
    grad_amount = float(1.00/instances)
    loop = 0
    while loop<=instances:
        if loop == 0:
            list_of_pos = [grad_position]
        else:
            list_of_pos.append(grad_position)
        grad_position += grad_amount
        loop += 1
    return list_of_pos

#######################################################################################################

def fn_clean_up_text(t_string):
    # replace newline characters from current text with space
    loop,t_flag = 0,0
    n_string = "No text has been entered!"
    while (loop<len(t_string)):
        if (t_string[loop]>" "):
            if (t_flag == 0):
                n_string = t_string[loop]
                t_flag = 1
            else:
                n_string += t_string[loop]
        else:
            if (t_flag>0):
                n_string += " "
                
        loop += 1
    return n_string

#######################################################################################################

def fn_shade_text_colour(f_colour, bl_colour, instances, amount):
    red_amount_old = f_colour[0]
    green_amount_old = f_colour[1]
    blue_amount_old = f_colour[2]
    red_amount_new = bl_colour[0]
    green_amount_new = bl_colour[1]
    blue_amount_new = bl_colour[2]
    zero = 0.0
	
    if red_amount_old<red_amount_new:
        red_spread = float(red_amount_new - red_amount_old)
        red_step = float(red_spread/instances)
    else:
        red_spread = float(red_amount_old - red_amount_new)
        red_step = float(zero - (red_spread/instances))
		
    if green_amount_old<green_amount_new:
        green_spread = float(green_amount_new - green_amount_old)
        green_step = float(green_spread/instances)
    else:
        green_spread = float(green_amount_old - green_amount_new)
        green_step = float(zero - float((green_spread/instances)))

    if blue_amount_old<blue_amount_new:
        blue_spread = float(blue_amount_new - blue_amount_old)
        blue_step = float(blue_spread/instances)
    else:
        blue_spread = float(blue_amount_old - blue_amount_new)
        blue_step = float(zero - float((blue_spread/instances)))

    new_red_amount = red_amount_old + (red_step * amount)
    if new_red_amount < 0: new_red_amount = 0
    if new_red_amount > 255: new_red_amount = 255
	
    new_green_amount = green_amount_old + (green_step * amount)
    if new_green_amount < 0: new_green_amount = 0
    if new_green_amount > 255: new_green_amount = 255
	
    new_blue_amount = blue_amount_old + (blue_step * amount)
    if new_blue_amount < 0: new_blue_amount = 0
    if new_blue_amount > 255: new_blue_amount = 255
	
    new_f_colour = (int(new_red_amount), int(new_green_amount), int(new_blue_amount))
	
    return new_f_colour
		
#######################################################################################################

def fn_shade_gradient_colour(colours_of_samples, instances):
    red_amount = colours_of_samples[(instances*4) + 0] * 255
    green_amount = colours_of_samples[(instances*4) + 1] * 255
    blue_amount = colours_of_samples[(instances*4) + 2] * 255
    alpha_amount = colours_of_samples[(instances*4) + 3] * 255

    if red_amount < 0: red_amount = 0
    if red_amount > 255: red_amount = 255
    if green_amount < 0: green_amount = 0
    if green_amount > 255: green_amount = 255
    if blue_amount < 0: blue_amount = 0
    if blue_amount > 255: blue_amount = 255
    if alpha_amount < 0: alpha_amount = 0
    if alpha_amount > 255: alpha_amount = 255
	
    new_f_colour = (int(red_amount), int(green_amount), int(blue_amount), int(alpha_amount))
    return new_f_colour

#######################################################################################################

def fn_remove_duplicate_spaces(joining_text_list, text):
    prev_joining_char,clean_joining_text, clean_plain_text = "","",""
    loop = 0
    while loop < len(joining_text_list):
        if loop > 0: 
            prev_joining_char = joining_text_list[loop-1]
        
        cur_joining_char = joining_text_list[loop]
        cur_plain_char = text[loop]
        if (prev_joining_char == " " and cur_joining_char == " "):
            dummy = 1
        else:
            clean_joining_text += cur_joining_char
            clean_plain_text += cur_plain_char
        loop += 1
    return clean_joining_text, clean_plain_text

#######################################################################################################

def fn_check_text_along_path(image, layer, joining_text_list, text, font_name_pointer, 
                        f_name, f_colour,  bl_colour, spread_colour, joinedup_gradient, use_gradient_colour, f_size, step_size, kern_amount, 
                        keep_layers, keep_paths, use_space_to_change_step_size, compensate_multiplier, 
                        use_text_size_modifier, initial_text_size, medial_text_size, final_text_size, position_of_medial, repeat_text, reversed_writing, Left_to_Right_path, rotate_text, user_angle, jitter, jitter_range, use_custom, xcffilename, fontimg):
    # Calculates the repeated text that will fit the text along the current path
    joining_text_list, text = fn_remove_duplicate_spaces(joining_text_list, text)
    repeat_stop = False
    governor = 0 # initialise variable to use to prevent creation of unnecessary layers
    attack = 0 # initialise variable to mark point of change in text size within text string when size modifier flag is set
    decay = 0
    spaces_loop = 0
    spaces_count = text.count(" ")
    if not use_gradient_colour:
        if spaces_count<2: spaces_count = 2
    target = 0
    original_font_colour = f_colour
    shading_loop = 0
    check_loop = 0
    check_text = ""
    width_of_text = 0
    if len(joining_text_list) % 2 == 0: # variable to help position attack (first) phase of change in text size
        even = True
    else:
        even = False
        
    # Get the current active path
    active_vector = pdb.gimp_image_get_active_vectors(image)
    active_vector_name = pdb.gimp_item_get_name(active_vector)
	
    if (active_vector == -1):
        active_vector = pdb.gimp_image_set_active_vectors(image,image.vectors[0])
	
    active_vectors = pdb.gimp_image_get_active_vectors(image)
    # Get path stroke information
    NumberOfStrokes_in_Path,ActivePath_StrokeIDs_List = pdb.gimp_vectors_get_strokes(active_vector)
    ActivePath_StrokeID = ActivePath_StrokeIDs_List[0]
    path_length = pdb.gimp_vectors_stroke_get_length(active_vectors,ActivePath_StrokeID,0.01)
	
	# initialise some variables
    original_f_size = f_size
    old_f_xpos = 0
    f_xpos = 0
    f_ypos = 40
    curve = 0
    curve_compensation = 0
    temp_angle_diff = 0
	
    #2.17 lines added
    if use_custom:
        file_dummy = 0
        #fontimg = pdb.gimp_xcf_load(file_dummy,xcffilename,xcffilename)
        space_name = " .png"
        char_name = " "
        layer_name = char_name + ".png"
        char_layer = pdb.gimp_image_get_layer_by_name(fontimg,layer_name)
        if char_layer < 0: #layer not found
            space_name = "space.png"
            char_name = "space"
            layer_name = char_name + ".png"
            char_layer = pdb.gimp_image_get_layer_by_name(fontimg,layer_name)
        if char_layer < 0: #layer not found
            space_name = "nospace"
            #pdb.gimp_message("No space character..." )
            space_width = f_size * 1.164 # Make up a space width for this font size
        if char_layer >= 0:
            space_width = char_layer.width * 1.164
            font_y_offset = (char_layer.height * 1.164)/2

        else:
		
            # Find the width to use for the space character
            (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname(" ", f_size, PIXELS, str(f_name))
            space_width = fontbox_width
	
            # Check to see if the space character has paths so that we can set a flag if necessary
            temp_space_string = " -"
            temp_space_layer = pdb.gimp_text_layer_new(image,temp_space_string,f_name,f_size,0)
            pdb.gimp_image_insert_layer(image,temp_space_layer,None,-1)
            temp = pdb.gimp_vectors_new_from_text_layer(image, temp_space_layer)
            space_vectors_path = pdb.gimp_image_insert_vectors(image,temp,None,-1)
            space_vectors = pdb.gimp_image_get_vectors_by_name(image, temp_space_string)
            # Get space character path stroke information
            NumberOfStrokes_in_Space_Path,Space_Path_StrokeIDs_List = pdb.gimp_vectors_get_strokes(space_vectors)
            pdb.gimp_image_remove_vectors(image,space_vectors)
            temp = pdb.gimp_image_remove_layer(image, temp_space_layer)

            # Check to see if the font uses a (visible) glyph in the space character to set a flag
            space_has_vectors = False
            if (NumberOfStrokes_in_Space_Path > 1):
                if (font_name_pointer > -1): space_has_vectors = True

    text_layer_y_origin = image.height - (f_size + f_ypos)
	
    # Set the initial font size to use for text trails
    initial_f_size = f_size
    if initial_f_size < 2: initial_f_size = 2
    if initial_f_size > 3000: initial_f_size = 3000
    
    #2.17 lines added
    if not use_custom:
        (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname("dummy", initial_f_size, PIXELS, str(f_name))
        font_y_offset = fontbox_ascent

    char_width = 0
    # Calculate kerning to be used between characters, if required by user, based on size of font taken from user settings
    kern_factor = kern_amount/f_size
    kern_gap = kern_factor * f_size
    old_repeat_width = 0
    repeat_width = 0
    tempbox_width = 0
    NextChr_width = 0
    loop = 0
    remaining_path = 0
    no_of_repeats = -1
    rgb = 220
    destination_text, plain_text = "", ""
    original_len = len(joining_text_list)
    original_step_size = step_size
    original_use_text_size_modifier = use_text_size_modifier
	
    if " " not in joining_text_list: use_space_to_change_step_size = False # Force option off, if there are no space characters in the text
    original_use_space_to_change_step_size = use_space_to_change_step_size

    path_length_test = f_xpos + NextChr_width + (NextChr_width/2)
    c_loop = 0
    string_width = [0.0]
	
    #2.17 line added
    if use_custom: 
        font_name_pointer = -1
        space_has_vectors = False
    while (path_length_test < path_length) and not repeat_stop:
        remainder = loop % original_len
        destination_text += joining_text_list[remainder]
        plain_text += text[remainder]
        governor = 0
        target = len(destination_text)
        f_xpos = 0
        c_f_xpos = 0
        spaces_loop = 0
        spaces_count = destination_text.count(" ")            
				 	
        if len(destination_text) % 2 == 0: # variable to help position attack (first) phase of change in text size
            even = True
        else:
            even = False	
			
        use_space_to_change_step_size = original_use_space_to_change_step_size
        if " " not in destination_text: use_space_to_change_step_size = False # Force option off, if there are no space characters in the text

					
        if use_text_size_modifier:
            if use_space_to_change_step_size:  
                target = destination_text.count(" ")
                if target<2: #New value - was 3
                    use_space_to_change_step_size = False
                    target = len(destination_text)
                    use_text_size_modifier = False #New line
                else:
                    if target % 2 == 0: 
                        even = True
                    else:
                        even = False
            else:
                target = len(destination_text)
				
                if target<3: use_text_size_modifier = False		
		
        if target<3: 
            use_text_size_modifier = False
        else:
            if original_use_text_size_modifier == True: use_text_size_modifier = True
		
        if use_text_size_modifier:
            f_size = initial_text_size
            if target == 3: # ie minimum number of characters/spaces to work with
                attack = 1
            else:
                attack = int((float(target)/100) * int(position_of_medial)) # set attack to the point in the text string/space count where the font size will change
                if attack>(target - 1): attack = (target - 1)
        else:
            f_size = initial_f_size
			
				
        if use_text_size_modifier: # get step size for attack phase
            size_difference,new_step_size = 0,0
            if medial_text_size > initial_text_size: size_difference = medial_text_size - initial_text_size
            if initial_text_size > medial_text_size: size_difference = initial_text_size - medial_text_size
            if target>3:
                if use_space_to_change_step_size:
                    if attack < 1: attack = 1
                else:
                    if even: attack -= 1
                    if attack < 1: attack = 1
                new_step_size = size_difference/(attack)
            else:
                new_step_size = size_difference/(attack)
			
            if medial_text_size < initial_text_size: new_step_size = 0 - new_step_size
			
            step_size = new_step_size
			
        last_loop = 0
        iloop = 0
        # reset character x position adjustment factors
        tempbox_width, xpos_adjustment = 0, 0
        f_xpos_adjustment, j_xpos_adjustment, p_xpos_adjustment, t_xpos_adjustment = 0,0,0,0
        NextChr_width = 0
        z = 0
		
        initial_f_size = f_size		
        if initial_f_size < 2: initial_f_size = 2
        if initial_f_size > 3000: initial_f_size = 3000
		
        # Add visual indicator to show the filter is working for repeated text
        if loop % 3 == 0:
            working_text = "Working ."
        elif loop % 3 == 1:
            working_text = "Working .."
        else:
            working_text = "Working ..."        
        rgb -= 5
        if rgb < 0: rgb = 220
        working_text_colour = gimpcolor.RGB(rgb, rgb, rgb) 
        working_layer = pdb.gimp_text_layer_new(image,working_text,f_name,100,0)
        pdb.gimp_image_insert_layer(image,working_layer,None,-1)
        pdb.gimp_text_layer_set_color(working_layer, working_text_colour)
        pdb.gimp_layer_translate(working_layer,100,100)
        temp = pdb.gimp_displays_flush()
						
        if use_text_size_modifier: f_size = initial_text_size
		
		
        while (iloop<(len(destination_text) -1 ) and (path_length_test < path_length)):
		
            CurrentChar, RealCurrentChar = destination_text[iloop], plain_text[iloop]
		
            if iloop < (len(destination_text) - 1):
                NextChar, RealNextChar = destination_text[iloop+1], plain_text[iloop+1]
            else:
                NextChar, RealNextChar = "", ""
			

            if (font_name_pointer > -1):
                # Adjust kerning
                xpos_adjustment += fn_adjust_xpos(NextChar, CurrentChar, RealCurrentChar, RealNextChar, font_name_pointer, f_size, destination_text[iloop])

            if f_size <2: f_size = 2
            if f_size > 3000: f_size = 3000
			
            CurrChrWidth = 0
            ####### ####### ####### ####### ####### ####### 
            old_f_xpos = f_xpos
			
            #2.17 lines added
            if use_custom:
                file_dummy = 0
                #fontimg = pdb.gimp_xcf_load(file_dummy,xcffilename,xcffilename)
                custom_layer_name = CurrentChar + ".png"
                char_layer = pdb.gimp_image_get_layer_by_name(fontimg, custom_layer_name)
                
                if char_layer < 0: #layer not found
                    if CurrentChar == " ":
                        custom_layer_name = "space.png"
                        #char_layer = pdb.gimp_image_get_layer_by_name(fontimg, custom_layer_name)
                    else:
                        char = CurrentChar
                        if char.lower() == char:
                            char = char.upper()
                        else:
                            char = char.lower()
                        custom_layer_name = char + ".png"
                    char_layer = pdb.gimp_image_get_layer_by_name(fontimg, custom_layer_name)
                # work out scale of character 
                custom_scale = fn_get_custom_font_scale(f_size, char_layer.height)
                # sort out scaling to get the font size we need
                custom_width = char_layer.width * custom_scale # Get width of the current character
                #custom_height = copy.height * custom_scale
                ####### ####### ####### ####### ####### ####### 
                f_xpos = f_xpos + custom_width + kern_gap
                ####### ####### ####### ####### ####### ####### 
                CurrChrWidth = custom_width
            else:
                
                (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname(CurrentChar, f_size, PIXELS, str(f_name)) # Get width of the current character
                ####### ####### ####### ####### ####### ####### 
                f_xpos = f_xpos + fontbox_width + kern_gap
                ####### ####### ####### ####### ####### ####### 
                CurrChrWidth = fontbox_width
			

            if (CurrentChar == " "):
                if use_custom:
                    layer_name = " .png"
                    space_name = layer_name
                    char_layer = pdb.gimp_image_get_layer_by_name(fontimg,layer_name)
                    if char_layer < 0: #layer not found
                        layer_name = "space.png"
                        space_name = layer_name
                        char_layer = pdb.gimp_image_get_layer_by_name(fontimg,layer_name)
                        if char_layer < 0: #layer still not found
                            space_width = f_size * 1.164 # Make up a space width for this font size
                            font_y_offset = (f_size * 1.164)/2 # make this up too
                        else:
                            space_width = char_layer.width * 1.164
                            font_y_offset = (char_layer.height * 1.164)/2
				
                else:
                    # Find the width to use for the space character
                    (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname(" ", f_size, PIXELS, str(f_name))
                    space_width = fontbox_width
				
                f_size_adjustment = step_size
                f_size = f_size + f_size_adjustment
                if (f_size<2):
                    f_size = 2
                    governor += 1
                    if governor > 2: repeat_stop = True
                if (f_size>3000):
                    f_size = 3000
                    governor += 1
                    if governor > 2: repeat_stop = True
					
                kern_gap += (f_size * kern_factor)
                shading_loop += 1

                if use_text_size_modifier: # check to see if font size modifier is being used to begin changing size
                    if use_space_to_change_step_size:
                        if attack == spaces_loop:
                            decay = target - attack
							
                            if not even: decay -= 1     
                            if decay < 1: decay = 1
							
                            size_difference,new_step_size = 0,0
							
                            if medial_text_size > final_text_size: size_difference = medial_text_size - final_text_size
                            if final_text_size > medial_text_size: size_difference = final_text_size - medial_text_size
							
                            if target>3:
                                new_step_size = size_difference/decay
                            else:
                                if decay>1:
                                    new_step_size = size_difference/(decay-1)
                                else:
                                    new_step_size = size_difference/(decay)
									
                            if even: 
                                if final_text_size > medial_text_size:
						            f_size = medial_text_size + new_step_size
                                else:
                                    f_size -= (step_size + (new_step_size * 1))
									
                            if not even: f_size -= (step_size)
															
                            if final_text_size < medial_text_size: new_step_size = 0 - new_step_size
							
                            step_size = new_step_size
                        spaces_loop += 1
                    else:
				
                        if attack == iloop:
                            decay = target - attack
                            if even:
                                decay -=2
                            else:
                                decay -= 0
															
                            if decay < 1: decay = 1
                            size_difference,new_step_size = 0,0
							
                            if medial_text_size > final_text_size: size_difference = medial_text_size - final_text_size
                            if final_text_size > medial_text_size: size_difference = final_text_size - medial_text_size
							
                            if target>3:
                                if decay>1:
                                    if even:
                                        new_step_size = size_difference/(decay)
                                    else:
                                        new_step_size = size_difference/(decay-1)
                                else:
                                    new_step_size = size_difference/(decay)
                            else:
                                if decay>1:
                                    new_step_size = size_difference/(decay-1)
                                else:
                                    new_step_size = size_difference/(decay)
							
                            if not even: 
                                if final_text_size > medial_text_size:
						            f_size = medial_text_size + new_step_size
                                
                                else:
                                    f_size -= (step_size + (new_step_size * 1))
						   
                            if even: f_size -= (step_size)
								
                            if new_step_size < 0.5: new_step_size = 0.5
                            if final_text_size < medial_text_size: new_step_size = 0 - new_step_size
                            step_size = new_step_size
						

            else:
                if (use_space_to_change_step_size == 0):
                
                    f_size_adjustment = step_size
                    f_size = f_size + f_size_adjustment
                    if (f_size<2):
                        f_size = 2
                        governor += 1
                        if governor > 2: 
                            repeat_stop = True
                    if (f_size>3000):
                        f_size = 3000
                        governor += 1
                        if governor > 2: repeat_stop = True
                    if use_text_size_modifier: # check to see if font size modifier is being used to begin changing size
                        if attack == iloop:
                            decay = len(destination_text) - attack
                            if even: 
                                decay -=2 
                            else:
                                decay -= 0
							
                            if decay < 1: decay = 1
                            size_difference,new_step_size = 0,0
						
                            if medial_text_size > final_text_size: size_difference = medial_text_size - final_text_size
                            if final_text_size > medial_text_size: size_difference = final_text_size - medial_text_size
						
                            if len(destination_text)>3:
                                if decay>1:
                                    if even:
                                        new_step_size = size_difference/(decay)
                                    else:
                                        new_step_size = size_difference/(decay-1)
                                else:
                                    new_step_size = size_difference/(decay)
                            else:
                                if decay>1:
                                    new_step_size = size_difference/(decay-1)
                                else:
                                    new_step_size = size_difference/(decay)
							
                            if not even: 
                                if final_text_size > medial_text_size:
						            f_size = medial_text_size + new_step_size
                                else:
                                    f_size -= (step_size + (new_step_size * 1))
						   
                            if even: f_size -= (step_size)
                            if (f_size<2): f_size = 2
                            if (f_size>3000): f_size = 3000
                            if final_text_size < medial_text_size: new_step_size = 0 - new_step_size
                            step_size = new_step_size
                    kern_gap = (f_size * kern_factor)
					
            NextChr_width = 0
            if use_text_size_modifier:
                if not even: 
                    if final_text_size > medial_text_size:
                        temp_f_size = f_size + new_step_size
                    else:
                        temp_f_size = f_size - new_step_size
                else:
                    if final_text_size > medial_text_size:
                        temp_f_size = f_size + new_step_size
                    else:
                        temp_f_size = f_size - new_step_size
					
                if (temp_f_size<2): temp_f_size = 2
                if (temp_f_size>3000): temp_f_size = 3000
                if use_custom:
                    custom_layer_name = NextChar + ".png"
                    char_layer = pdb.gimp_image_get_layer_by_name(fontimg,layer_name)
                    if char_layer < 0: #layer not found
                        if NextChar == " ":
                            custom_layer_name = "space.png"
                            #char_layer = pdb.gimp_image_get_layer_by_name(fontimg, custom_layer_name)
                        else:
                            char = NextChar
                            if char.lower() == char:
                                char = char.upper()
                            else:
                                char = char.lower()
                            custom_layer_name = char + ".png"
                        char_layer = pdb.gimp_image_get_layer_by_name(fontimg, custom_layer_name)
                    # work out scale of character 
                    custom_scale = fn_get_custom_font_scale(f_size, char_layer.height)
                    # sort out scaling to get the font size we need
                    NextChr_width = char_layer.width * custom_scale				
                else:
                    (NextChr_width, NextChr_height, NextChr_ascent, NextChr_descent) = pdb.gimp_text_get_extents_fontname(NextChar, temp_f_size, PIXELS, str(f_name))
                
            else:
                if use_custom:                    
                    custom_layer_name = NextChar + ".png"
                    char_layer = pdb.gimp_image_get_layer_by_name(fontimg,layer_name)
                    if char_layer < 0: #layer not found
                        if NextChar == " ":
                            custom_layer_name = "space.png"
                            #char_layer = pdb.gimp_image_get_layer_by_name(fontimg, custom_layer_name)
                        else:
                            char = NextChar
                            if char.lower() == char:
                                char = char.upper()
                            else:
                                char = char.lower()
                            custom_layer_name = char + ".png"
                        char_layer = pdb.gimp_image_get_layer_by_name(fontimg, custom_layer_name)
                    # work out scale of character 
                    custom_scale = fn_get_custom_font_scale(f_size, char_layer.height)
                    # sort out scaling to get the font size we need
                    NextChr_width = char_layer.width * custom_scale	
                else:
                    (NextChr_width, NextChr_height, NextChr_ascent, NextChr_descent) = pdb.gimp_text_get_extents_fontname(NextChar, f_size, PIXELS, str(f_name))
				
            z = f_xpos + NextChr_width
            ####### ####### ####### ####### ####### 
            path_length_test = f_xpos
            ####### ####### ####### ####### #######
            iloop += 1        
				
        if loop % original_len == 0:

            check_repeat = old_f_xpos # holds width of previous string 
            old_remaining_path = remaining_path # holds previous remaining amount of room along the path
            remaining_path = path_length - f_xpos # holds current remaining amount of room along the path
            old_repeat_width = repeat_width # holds previous repeated string width
            repeat_width = f_xpos # holds current repeated string width
            no_of_repeats = loop // original_len		
        
        pdb.gimp_image_remove_layer(image, working_layer)
        loop += 1
        last_loop = loop
			
        if not repeat_text:
            if loop // original_len > 0: repeat_stop = True
		
    temp_plain_text = plain_text
    if repeat_text:
        if no_of_repeats < 1: no_of_repeats = 1
        check_text = joining_text_list * no_of_repeats
        plain_text = text * no_of_repeats
    else:		
        check_text = destination_text
			
    temp_plain_repeated = text * (loop // original_len)
    return check_text, plain_text, check_loop, old_repeat_width, path_length, path_length - repeat_width

#######################################################################################################

#### END OF fn_check_text_along_path

#######################################################################################################

def proc_text_along_path(image, layer, joining_text_list, text, font_name_pointer, 
                        f_name, f_colour,  bl_colour, spread_colour, joinedup_gradient, use_gradient_colour, f_size, step_size, kern_amount, 
                        keep_layers, keep_paths, use_space_to_change_step_size, compensate_multiplier, 
                        use_text_size_modifier, initial_text_size, medial_text_size, final_text_size, 
                        position_of_medial, final_temp_text, repeat_text, remaining_path, reversed_writing, Left_to_Right_path, rotate_text, user_angle, jitter, jitter_range, tranparency, use_custom, xcffilename, fontimg):

				
    # Creates text layers
    initial_step_size = step_size
    label_details = fn_label_characters(text, 12)
    label_fontsize_info = " " + str(int(f_size)) + "pt"
    
    original_user_angle = user_angle
    governor = 0 # initialise variable to use to prevent creation of unnecessary layers
    attack, decay = 0, 0 # initialise variable to mark point of change in text size within text string when size modifier flag is set
    spaces_loop = 0
    spaces_count = text.count(" ")
	
    if spaces_count > 0:
        if use_space_to_change_step_size:
            if text.endswith(" "): 
                padding = remaining_path / (spaces_count + 0)
            else:
                if remaining_path > 40:
                    padding = remaining_path / (spaces_count + 1)
                else:
                    padding = 0
        else:
            if text.endswith(" "):
                padding = remaining_path / (spaces_count + 0)
            else:
                padding = remaining_path / (spaces_count + 1)
            
    else:
        if font_name_pointer > -1:
            padding = 0
        else:
            if text.endswith(" "):
                padding = remaining_path / len(joining_text_list)
            else:
                padding = remaining_path / (len(joining_text_list) + 1)
						
    if not use_gradient_colour:
        if spaces_count<2: spaces_count = 2
    target = 0
    original_font_colour = f_colour
    shading_loop = 0
	
    if len(text) % 2 == 0: # variable to help position attack (first) phase of change in text size
        even = True
    else:
        even = False
		
    if " " not in text: use_space_to_change_step_size = False # Force option off, if there are no space characters in the text

    if use_text_size_modifier:
        if use_space_to_change_step_size:  
            target = text.count(" ")
            if target<2:
                use_space_to_change_step_size = False
                target = len(text)
                use_text_size_modifier = False #New line
            else:
                if target % 2 == 0:
                    even = True
                else:
                    even = False
        else:
            target = len(text)
            if target<3: use_text_size_modifier = False

    if use_gradient_colour: 
        # get number of colours etc needed for using colours from current ui gradient
        current_gradient = joinedup_gradient
        if use_space_to_change_step_size:
            list_of_gradient_positions = fn_get_gradient_positions(spaces_count)
            number_of_samples, colours_of_samples = pdb.gimp_gradient_get_custom_samples(current_gradient, len(list_of_gradient_positions), list_of_gradient_positions,False)
        else:
            list_of_gradient_positions = fn_get_gradient_positions(len(final_temp_text))
            number_of_samples, colours_of_samples = pdb.gimp_gradient_get_custom_samples(current_gradient, len(list_of_gradient_positions), list_of_gradient_positions,False)

    if use_text_size_modifier:
        f_size = initial_text_size
        if target == 3: # ie minimum number of characters/spaces to work with
            attack = 1
        else:
            f_size = initial_text_size
            attack = int((float(target)/100) * int(position_of_medial)) # set attack to the point in the text string/space count where the font size will change
            if attack>(target - 1): attack = (target - 1)
	
    if use_text_size_modifier: # get font step size for attack phase
        size_difference,new_step_size = 0,0
        if medial_text_size > initial_text_size: size_difference = medial_text_size - initial_text_size
        if initial_text_size > medial_text_size: size_difference = initial_text_size - medial_text_size
        if target>3:
            if not use_space_to_change_step_size: 
                if even: attack -= 1
            if attack < 1: attack = 1
            new_step_size = size_difference/(attack)
        else:
            new_step_size = size_difference/(attack)
			
        #if new_step_size < 0.5: new_step_size = 0.5
        if medial_text_size < initial_text_size: new_step_size = 0 - new_step_size
        step_size = new_step_size
        
    # Get the current active path
    active_vector = pdb.gimp_image_get_active_vectors(image)
    active_vector_name = pdb.gimp_item_get_name(active_vector)
	
    if (active_vector == -1):
        active_vector = pdb.gimp_image_set_active_vectors(image,image.vectors[0])
	
    active_vectors = pdb.gimp_image_get_active_vectors(image)
    # Get path stroke information
    NumberOfStrokes_in_Path,ActivePath_StrokeIDs_List = pdb.gimp_vectors_get_strokes(active_vector)
    ActivePath_StrokeID = ActivePath_StrokeIDs_List[0]
    path_length = pdb.gimp_vectors_stroke_get_length(active_vectors,ActivePath_StrokeID,0.01)
	
	# initialise some variables
    f_xpos = 0
    f_ypos = 40
    curve = 0
    curve_compensation = 0
    temp_angle_diff = 0
	
	
    #2.17 lines added
    if use_custom:
        
        file_dummy = 0
        #fontimg = pdb.gimp_xcf_load(file_dummy,xcffilename,xcffilename)
        space_name = " .png"
        char_name = " "
        layer_name = char_name + ".png"
        char_layer = pdb.gimp_image_get_layer_by_name(fontimg,layer_name)
        if char_layer < 0: #layer not found
            space_name = "space.png"
            char_name = "space"
            layer_name = char_name + ".png"
            char_layer = pdb.gimp_image_get_layer_by_name(fontimg,layer_name)
        if char_layer < 0: #layer not found
            space_name = "nospace"
            #pdb.gimp_message("No space character..." )
            space_width = f_size * 1.164 # Make up a space width for this font size
        if char_layer >= 0:
            space_width = char_layer.width * 1.164
            font_y_offset = (char_layer.height * 1.164)/2
    else:
	
        # Find the width to use for the space character
        (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname(" ", f_size, PIXELS, str(f_name))
        space_width = fontbox_width
	
        # Check to see if the space character has paths so that we can set a flag if necessary
        temp_space_string = " -"
        temp_space_layer = pdb.gimp_text_layer_new(image,temp_space_string,f_name,f_size,0)
        pdb.gimp_image_insert_layer(image,temp_space_layer,None,-1)
        temp = pdb.gimp_vectors_new_from_text_layer(image, temp_space_layer)
        space_vectors_path = pdb.gimp_image_insert_vectors(image,temp,None,-1)
        space_vectors = pdb.gimp_image_get_vectors_by_name(image, temp_space_string)
        # Get space character path stroke information
        NumberOfStrokes_in_Space_Path,Space_Path_StrokeIDs_List = pdb.gimp_vectors_get_strokes(space_vectors)
        pdb.gimp_image_remove_vectors(image,space_vectors)
        temp = pdb.gimp_image_remove_layer(image, temp_space_layer)

        # Check to see if the font uses a (visible) glyph in the space character to set a flag
        space_has_vectors = False
        if (NumberOfStrokes_in_Space_Path > 1):
            if (font_name_pointer > -1): space_has_vectors = True

    text_layer_y_origin = image.height - (f_size + f_ypos)
			
    initial_f_size = f_size		
    if initial_f_size < 2: initial_f_size = 2
    if initial_f_size > 3000: initial_f_size = 3000
    
    #2.17 lines added
    if not use_custom:
        (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname("dummy", initial_f_size, PIXELS, str(f_name))
        font_y_offset = fontbox_ascent

    char_width, NextChr_width = 0,0
    # Calculate kerning to be used between characters, if required by user, based on size of font taken from user settings
    kern_factor = kern_amount/f_size
    kern_gap = kern_factor * f_size
    tempbox_width = 0
    loop = 0
	
    #2.17 line added
    if use_custom: 
        font_name_pointer = -1
        space_has_vectors = False
	
    # Initialise a loop specifically to use for creating layer and path identifiers and names
    layer_and_path_loop = 0
			        
    while loop < len(joining_text_list) and (f_xpos + tempbox_width < path_length):
        CurrentChar, RealCurrentChar = joining_text_list[loop], text[loop]

        if loop > 0:		        
            if " " not in joining_text_list:
                f_xpos += padding
            else:
                # Find the width to use for the space character
                if CurrentChar == " ":
                    if use_custom:
                        char_layer = pdb.gimp_image_get_layer_by_name(fontimg,space_name)
                        custom_scale = fn_get_custom_font_scale(f_size, char_layer.height)
                        space_width = char_layer.width * custom_scale
                    else:
                        (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname(" ", f_size, PIXELS, str(f_name))
                        space_width = fontbox_width
                    f_xpos += padding
        if loop < (len(joining_text_list)-1):
            NextChar, RealNextChar = joining_text_list[loop + 1], text[loop + 1]
        else:
            NextChar, RealNextChar = "", ""
			
        # reset character x position adjustment factors
        f_xpos_adjustment, j_xpos_adjustment, p_xpos_adjustment, t_xpos_adjustment = 0,0,0,0
        xpos_adjustment = 0

        if (font_name_pointer > -1):
            # Adjust kerning
            if reversed_writing:
                if Left_to_Right_path:
                    xpos_adjustment += fn_adjust_xpos(NextChar, CurrentChar, RealCurrentChar, RealNextChar, font_name_pointer, f_size, joining_text_list[loop])
				 
                else:
                    
                    xpos_adjustment += fn_adjust_xpos(NextChar, CurrentChar, RealCurrentChar, RealNextChar, font_name_pointer, f_size, joining_text_list[loop])
            else:
                xpos_adjustment += fn_adjust_xpos(NextChar, CurrentChar, RealCurrentChar, RealNextChar, font_name_pointer, f_size, joining_text_list[loop])
				
            #pdb.gimp_message("xpos_adjustment = " + str(xpos_adjustment) + " CurrentChar = " + CurrentChar + " NextChar = " + NextChar + " RealCurrentChar = " + RealCurrentChar + " RealNextChar = " + RealNextChar)
				
        if (CurrentChar > " " or (CurrentChar == " " and space_has_vectors == True)):
            custom_layer_name = CurrentChar + ".png"
            # Create layers for each character in user defined string		        
            if (loop == 0):
				
                if use_custom:
                    # get id of character from custom font file in memory
                    char_layer = pdb.gimp_image_get_layer_by_name(fontimg, custom_layer_name)
                    if char_layer < 0: #if letter not found try swapping upper/lower case to look for
                        char = CurrentChar
                        if char.lower() == char:
                            char = char.upper()
                        else:
                            char = char.lower()
                        custom_layer_name = char + ".png"
                        char_layer = pdb.gimp_image_get_layer_by_name(fontimg, custom_layer_name)
                    # work out scale of character 
                    custom_scale = fn_get_custom_font_scale(f_size, char_layer.height)
                    # create, insert and set visibility the copy of the character layer into our image
                    copy = pdb.gimp_layer_new_from_drawable(char_layer,image)
                    pdb.gimp_image_insert_layer(image,copy,None,0)
                    pdb.gimp_item_set_visible(copy,True)
                    # sort out scaling to get the font size we need
                    custom_width = char_layer.width * custom_scale
                    custom_height = copy.height * custom_scale
                    old_interpolation = pdb.gimp_context_get_interpolation() # Note current interpolation method
                    pdb.gimp_context_set_interpolation(3) # Ensure that interpolation is set to Sinc (Lancos 3) method
                    pdb.gimp_layer_scale(copy,custom_width,custom_height,False)
                    pdb.gimp_context_set_interpolation(old_interpolation) # Restore old interpolation method
                    pdb.gimp_layer_set_offsets(copy,0,0)
                    # Create a list to hold the id of the first character
                    layer_id_list = [copy]
					
                    # Create a list to hold the layer name
                    layer_name_list = [CurrentChar] # Was: layer_name_list = [custom_layer_name]
					
                    # Set the new custom layer colour from the current font colour and the colour to shade to
                    if spread_colour and not use_gradient_colour: 
                        if not use_space_to_change_step_size:
                            custom_colour = original_font_colour
                        else:
                            custom_colour = original_font_colour
                        
                        pdb.plug_in_colorify(image, layer_id_list[layer_and_path_loop], custom_colour)
                    # Set the new text layer colour from the current gradient sampled colour
                    if use_gradient_colour:
                        custom_colour = fn_shade_gradient_colour(colours_of_samples,loop)
                        pdb.plug_in_colorify(image, layer_id_list[layer_and_path_loop], custom_colour)
                else:
                    # Create a list to hold the text layer identifiers
                    layer_id_list = ["text_layer" + str(layer_and_path_loop)]
                    # Create a list to hold the text layer names
                    layer_name_list = [str(joining_text_list[layer_and_path_loop])]
				
                    # Create the layer for the first character
                    layer_id_list[layer_and_path_loop] = pdb.gimp_text_layer_new(image,CurrentChar,f_name,f_size,0)
				
                    # and insert this first layer into the image
                    pdb.gimp_image_insert_layer(image,layer_id_list[layer_and_path_loop],None,-1)
				
                    # Set the new text layer colour
                    if use_gradient_colour: f_colour = fn_shade_gradient_colour(colours_of_samples,loop)
                    pdb.gimp_text_layer_set_color(layer_id_list[layer_and_path_loop], f_colour)
					
                # Set layer opacity as per user setting
                pdb.gimp_layer_set_opacity(layer_id_list[layer_and_path_loop], tranparency)
				
                # Rename the new layer 
                pdb.gimp_item_set_name(layer_id_list[layer_and_path_loop],str(text[loop]))

                # Set the x,y position of the first layer within the image
                pdb.gimp_layer_translate(layer_id_list[layer_and_path_loop],f_xpos,text_layer_y_origin)

                # note the width of the current character
                char_width = layer_id_list[layer_and_path_loop].width
				
                offset_x,offset_y = pdb.gimp_drawable_offsets(layer_id_list[layer_and_path_loop])

                # To insert paths of letters in the text list
                if use_custom:
                    dummy = 0
                    # get the id of the path of the layer from the custom font
                    char_vector = pdb.gimp_image_get_vectors_by_name(fontimg,custom_layer_name)
                    # Create a list to hold the path layer ids
                    path_id_list = [char_vector]
                    # Create another list to hold the names of the paths
                    paths_names_list = [str(CurrentChar)]
                else:
                    # Create a list to hold the path layer ids
                    path_id_list = [pdb.gimp_vectors_new_from_text_layer(image,layer_id_list[layer_and_path_loop])]
                    # Create another list to hold the names of the paths
                    paths_names_list = [str(joining_text_list[layer_and_path_loop])]

                    # Add the path to the image
                    temp = pdb.gimp_image_insert_vectors(image,path_id_list[layer_and_path_loop],None,-1)
                    # Get the path id associated with the current character
                    char_vector = pdb.gimp_image_get_vectors_by_name(image, paths_names_list[layer_and_path_loop])
                    
					
                if char_vector >=0:
                    # Get path info for current character path to work out character width				
                    NumberOfStrokes_in_Path,Char_Vector_StrokeIDs_List = pdb.gimp_vectors_get_strokes(char_vector)
                    Char_Vector_Stroke = Char_Vector_StrokeIDs_List[0]
                    TypeOfStrokeInChar_Vector,NumberOfPointsInChar_Vector,ListOfControlPoints,IsStrokeClosed = pdb.gimp_vectors_stroke_get_points(char_vector,Char_Vector_Stroke)
                    firstX = ListOfControlPoints[2]
                    secondX = ListOfControlPoints[len(ListOfControlPoints)-4]
                    minX = min(firstX,secondX)
                    maxX = max(firstX,secondX)
                    vector_char_width = maxX - minX #overwrite char_width to be used as character width.
                    add_width = vector_char_width
                    offsetX = minX-offset_x #overwrite offsetX with difference of starting X(minX) offset_x.
                    if use_custom: custom_width = add_width * custom_scale

                dist =  f_xpos + float(char_width)/2 #calculate center dist of letter before adding to width.
                # Get the x,y position of the required point along the active path to position the characters
                x_point,y_point,SlopeAtThisPoint,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist,0.01)
                x_point1,y_point1,SlopeAtThisPoint1,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist-5,0.01)
                x_point2,y_point2,SlopeAtThisPoint2,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist+5,0.01)

                translate_x = x_point - float(char_width)/2
                translate_y = y_point - float(layer_id_list[layer_and_path_loop].height)/2 # ie on the path
                pdb.gimp_layer_translate(layer_id_list[layer_and_path_loop],translate_x - f_xpos,translate_y - text_layer_y_origin)
                
                if rotate_text:
                    if jitter: 
                        user_angle = fn_angle_jitter(original_user_angle,jitter_range)
                        angle = math.radians(user_angle)
                    else:
                        angle = math.radians(original_user_angle)
                else:
                    if jitter:
                        angle = computeThetaWithSlope([x_point1,y_point1], [x_point2,y_point2],SlopeAtThisPoint)
                        jitter_angle = math.degrees(angle) 
                        jitter_angle = fn_angle_jitter(jitter_angle,jitter_range)
                        angle = math.radians(jitter_angle)
                    else:
                        angle = computeThetaWithSlope([x_point1,y_point1], [x_point2,y_point2],SlopeAtThisPoint)

                # Initialise angle variables to be used in trying to compensate for distortions along curved path
                current_angle, prev_angle = math.degrees(angle), math.degrees(angle)
				
                x,y = pdb.gimp_drawable_offsets(layer_id_list[layer_and_path_loop])
                if reversed_writing:
                    if rotate_text:
                        rotated_angle = angle
                    else:
                        rotated_angle = angle + math.radians(180)
						
                    if Left_to_Right_path:
                        temp = pdb.gimp_item_transform_flip_simple(layer_id_list[layer_and_path_loop], 0, True, 0) # flips each character through 180 degrees on horzontal axis
                        temp = pdb.gimp_item_transform_rotate(temp,rotated_angle ,False,x+float(char_width)/2,y+float(temp.height/2))# then rotates each character by further 180 degrees
                    else:
                        temp = pdb.gimp_item_transform_flip_simple(layer_id_list[layer_and_path_loop], 0, True, 0) # flips each character through 180 degrees on horzontal axis
                        temp = pdb.gimp_item_transform_rotate(temp,rotated_angle ,False,x+float(char_width)/2,y+float(temp.height/2))# then rotates each character by further 180 degrees
                        
                else:
                    #v2.11 line
                    temp = pdb.gimp_item_transform_rotate(layer_id_list[loop],angle,False,x+float(char_width)/2,y+float(layer_id_list[layer_and_path_loop].height/2))
				
                curve_compensation = fn_compensate_for_curve_in_path(prev_angle, current_angle, f_size)
                if (keep_paths == 0):
                    if char_vector >=0 and not use_custom:
                        pdb.gimp_image_remove_vectors(image,path_id_list[layer_and_path_loop])
                layer_and_path_loop += 1

            else: # loop is greater than 0 i.e. there are subsequent characters to deal with
			
                if use_custom:
                    # get id of character from custom font file in memory
                    char_layer = pdb.gimp_image_get_layer_by_name(fontimg, custom_layer_name)
                    char_vector = pdb.gimp_image_get_vectors_by_name(fontimg,custom_layer_name)
                    if char_layer < 0: #if letter not found try swapping upper/lower case to look for
                        char = CurrentChar
                        if char.lower() == char:
                            char = char.upper()
                        else:
                            char = char.lower()
                        custom_layer_name = char + ".png"
                        char_layer = pdb.gimp_image_get_layer_by_name(fontimg, custom_layer_name)
                    # work out scale of character 
                    custom_scale = fn_get_custom_font_scale(f_size, char_layer.height)
                    # create, insert and set visibility the copy of the character layer into our image
                    copy = pdb.gimp_layer_new_from_drawable(char_layer,image)
                    pdb.gimp_image_insert_layer(image,copy,None,0)
                    pdb.gimp_item_set_visible(copy,True)
                    # sort out scaling to get the font size we need
                    custom_width = copy.width * custom_scale
                    custom_height = copy.height * custom_scale
                    old_interpolation = pdb.gimp_context_get_interpolation() # Note current interpolation method
                    pdb.gimp_context_set_interpolation(3) # Ensure that interpolation is set to Sinc (Lancos 3) method
                    pdb.gimp_layer_scale(copy,custom_width,custom_height,False)
                    pdb.gimp_context_set_interpolation(old_interpolation) # Restore old interpolation method
                    pdb.gimp_layer_set_offsets(copy,0,0)
                    # Create a list to hold the id of subsequent characters
                    # Create a list to hold subsequent layer names
                    #layer_name_list.append([custom_layer_name])
                    # Add further layer identifiers to the list of text layer identifiers
                    layer_id_list.append(copy)
                    custom_scale = fn_get_custom_font_scale(f_size, char_layer.height)
                    char_width = custom_width
                    # Set the new custom layer colour from the current font colour and the colour to shade to
                    if spread_colour and not use_gradient_colour: 
                        if not use_space_to_change_step_size:
                            custom_colour = fn_shade_text_colour(original_font_colour, bl_colour, len(text)-1, loop)
                        else:
                            custom_colour = fn_shade_text_colour(original_font_colour, bl_colour, spaces_count - 1, shading_loop)
                        pdb.plug_in_colorify(image, layer_id_list[layer_and_path_loop], custom_colour)
                    # Set the new custom layer colour from the current gradient sampled colour
                    if use_gradient_colour:
                        custom_colour = fn_shade_gradient_colour(colours_of_samples,loop)
                        pdb.plug_in_colorify(image, layer_id_list[layer_and_path_loop], custom_colour)
					
                else:
                    # Add further layer identifiers to the list of text layer identifiers
                    layer_id_list.append(["text_layer" + str(layer_and_path_loop)])
                    # and create each subsequent layer
                    layer_id_list[layer_and_path_loop] = pdb.gimp_text_layer_new(image,CurrentChar,f_name,f_size,0)
				
                    # Insert subsequent layers into the image
                    pdb.gimp_image_insert_layer(image,layer_id_list[layer_and_path_loop],None,-1)
				
                    # Set the new text layer colour
                    if spread_colour and not use_gradient_colour: 
                        if not use_space_to_change_step_size:
                            f_colour = fn_shade_text_colour(original_font_colour, bl_colour, len(text)-1, loop)
                        else:
                            f_colour = fn_shade_text_colour(original_font_colour, bl_colour, spaces_count - 1, shading_loop)
                    # Set the new text layer colour from the current gradient sampled colour
                    if use_gradient_colour: 
                        if use_space_to_change_step_size: 
                            f_colour = fn_shade_gradient_colour(colours_of_samples,shading_loop)
                        else:
                            f_colour = fn_shade_gradient_colour(colours_of_samples,loop)
						
                    pdb.gimp_text_layer_set_color(layer_id_list[layer_and_path_loop], f_colour)
				
                    # Get width of current character
                    char_width = layer_id_list[layer_and_path_loop].width
                # Set layer opacity as per user setting
                pdb.gimp_layer_set_opacity(layer_id_list[layer_and_path_loop], tranparency)
                # Rename the layer
                pdb.gimp_item_set_name(layer_id_list[layer_and_path_loop],str(text[loop]))
                # Get the name of the current layer
                if use_custom: pdb.gimp_item_set_name(layer_id_list[layer_and_path_loop],str(text[loop]))
                temp = pdb.gimp_item_get_name((layer_id_list[layer_and_path_loop]))

                # Add further layer names to the list of text layer names
                layer_name_list.append(temp)

                # Rename the layer to account for multiple occurences of the same letter
                pdb.gimp_item_set_name(layer_id_list[layer_and_path_loop],str(layer_name_list[layer_and_path_loop]))
                # Set x,y the position of subsequent layers in the image
				
                pdb.gimp_layer_translate(layer_id_list[layer_and_path_loop],f_xpos,text_layer_y_origin )

                if not use_custom:
                    # Add further path identifiers to the list of path identifiers
                    path_id_list.append(pdb.gimp_vectors_new_from_text_layer(image,layer_id_list[layer_and_path_loop]))
                    # Add further names of the list paths names
                    # Use loop instead of layer_and_path_loop since we are referencing the original string for the path layer 
                    paths_names_list.append(str(joining_text_list[loop]))
                    # Insert path
                    temp = pdb.gimp_image_insert_vectors(image,path_id_list[layer_and_path_loop],None,-1)
				
                    char_vector = pdb.gimp_image_get_vectors_by_name(image, paths_names_list[layer_and_path_loop])
				
                if char_vector >=0:
                    NumberOfStrokes_in_Path,Char_Vector_StrokeIDs_List = pdb.gimp_vectors_get_strokes(char_vector)
                    Char_Vector_Stroke = Char_Vector_StrokeIDs_List[0]
                    TypeOfStrokeInChar_Vector,NumberOfPointsInChar_Vector,ListOfControlPoints,IsStrokeClosed = pdb.gimp_vectors_stroke_get_points(char_vector,Char_Vector_Stroke)
                    firstX = ListOfControlPoints[2]
                    secondX = ListOfControlPoints[len(ListOfControlPoints)-4]
                    minX = min(firstX,secondX)
                    maxX = max(firstX,secondX)
                    vector_char_width = maxX - minX #overwrite char_width to be used as character width.
                    add_width = vector_char_width
                    offsetX = minX-offset_x #overwrite offsetX with difference of starting X(minX) offset_x.
                    if use_custom: custom_width = add_width * custom_scale

                dist =  f_xpos + float(char_width)/2 #calculate center dist of letter before adding to width.

                # Get the x,y position of the required point along the active path to position the characters
                x_point,y_point,SlopeAtThisPoint,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist,0.01)
                #x_point1,y_point1,SlopeAtThisPoint1,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist-5,0.01)
                #x_point2,y_point2,SlopeAtThisPoint2,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist+5,0.01)
                x_point1,y_point1,SlopeAtThisPoint1,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist,0.01)
                x_point2,y_point2,SlopeAtThisPoint2,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist+0.5,0.01)
                if rotate_text:
                    if jitter: 
                        user_angle = fn_angle_jitter(original_user_angle,jitter_range)
                        angle = math.radians(user_angle)
                    else:
                        angle = math.radians(original_user_angle)
                else:
                    if jitter:
                        angle = computeThetaWithSlope([x_point1,y_point1], [x_point2,y_point2],SlopeAtThisPoint)
                        jitter_angle = math.degrees(angle) 
                        jitter_angle = fn_angle_jitter(jitter_angle,jitter_range)
                        angle = math.radians(jitter_angle)
                    else:
                        angle = computeThetaWithSlope([x_point1,y_point1], [x_point2,y_point2],SlopeAtThisPoint)

                #angle = computeThetaWithSlope([x_point1,y_point1], [x_point2,y_point2],SlopeAtThisPoint)
                #angle = computeThetaWithSlope([x_point,y_point], [x_point,y_point],SlopeAtThisPoint)
                current_angle = math.degrees(angle)
								
                curve_compensation,temp_angle_diff = fn_compensate_for_curve_in_path(prev_angle, current_angle, f_size)
                curve_compensation = (curve_compensation * compensate_multiplier)
                curve = curve_compensation
				
                dist =  (f_xpos + curve_compensation) + float(char_width)/2 #calculate center dist of letter before adding to width.

                x_point,y_point,SlopeAtThisPoint,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist,0.01)
                x_point1,y_point1,SlopeAtThisPoint1,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist-5,0.01)
                x_point2,y_point2,SlopeAtThisPoint2,IsThisPointValid = pdb.gimp_vectors_stroke_get_point_at_dist(active_vectors,ActivePath_StrokeID,dist+5,0.01)
				
                translate_x = x_point - float(char_width)/2
                translate_y = y_point - float(layer_id_list[layer_and_path_loop].height)/2 # ie on the path
                pdb.gimp_layer_translate(layer_id_list[layer_and_path_loop],translate_x - f_xpos,translate_y - text_layer_y_origin)
												
                x,y = pdb.gimp_drawable_offsets(layer_id_list[layer_and_path_loop])
				
                if reversed_writing:
                    if rotate_text:
                        rotated_angle = angle
                    else:
                        rotated_angle = angle + math.radians(180)
						
                    if Left_to_Right_path:
						
                        temp = pdb.gimp_item_transform_flip_simple(layer_id_list[layer_and_path_loop], 0, True, 0) # flip original layer along horizontal axis
                        temp = pdb.gimp_item_transform_rotate(temp,rotated_angle ,False,x+float(char_width)/2,y+float(temp.height/2)) # then rotate flipped layer on path with extra 180 degrees
                    else:
                        temp = pdb.gimp_item_transform_flip_simple(layer_id_list[layer_and_path_loop], 0, True, 0) # flip original layer along horizontal axis
                        temp = pdb.gimp_item_transform_rotate(temp,rotated_angle ,False,x+float(char_width)/2,y+float(temp.height/2)) # then rotate flipped layer on path with extra 180 degrees
                else:
                    #v 2.11 line
                    temp = pdb.gimp_item_transform_rotate(layer_id_list[layer_and_path_loop],angle,False,x+float(char_width)/2,y+float(layer_id_list[layer_and_path_loop].height/2)) # rotate original layer on path
						
                if not use_custom:						
                    # Delete path
                    if (keep_paths == 0):
                        pdb.gimp_image_remove_vectors(image,path_id_list[layer_and_path_loop])
                # Merge new character layer to first character layer
                if (keep_layers == 0):
                    pdb.gimp_image_merge_down(image, layer_id_list[layer_and_path_loop],0)
                temp = pdb.gimp_displays_flush()

                layer_and_path_loop += 1				

        # Update y position adjustment
        if use_custom:
            text_layer_y_origin = text_layer_y_origin + (font_y_offset - (custom_height / 2))
    	    font_y_offset = (custom_height / 2)
            f_xpos = f_xpos + custom_width + kern_gap + curve - xpos_adjustment
        else:
            (fontbox_width, fontbox_height, fontbox_ascent, fontbox_descent) = pdb.gimp_text_get_extents_fontname(CurrentChar, f_size, PIXELS, str(f_name))
            text_layer_y_origin = text_layer_y_origin + (font_y_offset - fontbox_ascent)
            font_y_offset = fontbox_ascent
		
            f_xpos = f_xpos + fontbox_width + kern_gap + curve - xpos_adjustment
		
        prev_angle = current_angle

        if (CurrentChar == " "):
            
            f_size_adjustment = step_size
            f_size = f_size + f_size_adjustment
            # Stop font size becoming too small/large
            if (f_size<2):
                f_size = 2
                governor += 1
                if governor > 2: loop = len(joining_text_list)
            if (f_size>3000):
                f_size = 3000
                governor += 1
                if governor > 2: loop = len(joining_text_list)
				
            kern_gap += (f_size * kern_factor)
            shading_loop += 1
			
            if use_text_size_modifier: # check to see if font size modifier is being used to begin changing size
                if use_space_to_change_step_size:
				
                    if attack == spaces_loop:
                        decay = target - attack
                        if not even: decay -= 1
						
                        if decay < 1: decay = 1
                        size_difference,new_step_size = 0,0
						
                        if medial_text_size > final_text_size: size_difference = medial_text_size - final_text_size
                        if final_text_size > medial_text_size: size_difference = final_text_size - medial_text_size
						
                        if target>3:
                            new_step_size = size_difference/decay
                        else:
                            if decay>1:
                                new_step_size = size_difference/(decay-1)
                            else:
                                new_step_size = size_difference/(decay)
							
                        if even: 
                            if final_text_size > medial_text_size:
						        f_size = medial_text_size + new_step_size                                
                            else:
                                f_size -= (step_size + (new_step_size * 1))
						   
                        if not even: f_size -= step_size
						
                        if final_text_size < medial_text_size: new_step_size = 0 - new_step_size
                        step_size = new_step_size
						
                    spaces_loop += 1
					
                else:
				
                    if attack == loop:							
                        decay = target - attack
                        if even: 
                            decay -=2 
                        else:
                            decay -= 0							
                        if decay < 1: decay = 1
                        size_difference,new_step_size = 0,0
						
                        if medial_text_size > final_text_size: size_difference = medial_text_size - final_text_size
                        if final_text_size > medial_text_size: size_difference = final_text_size - medial_text_size
						
                        if target>3:
                            if decay>1:
                                if even:
                                    new_step_size = size_difference/decay
                                else:
                                    new_step_size = size_difference/(decay-1)
                            else:
                                new_step_size = size_difference/decay
                        else:
                            if decay>1:
                                new_step_size = size_difference/(decay-1)
                            else:
                                new_step_size = size_difference/decay
							
                        if not even: 
                            if final_text_size > medial_text_size:
						        f_size = medial_text_size + new_step_size
                                
                            else:
                                f_size -= (step_size + (new_step_size * 1))
						   
                        if even: f_size -= (step_size)
							
                        if final_text_size < medial_text_size: new_step_size = 0 - new_step_size
                        step_size = new_step_size
        else:
            if (use_space_to_change_step_size == 0):
                f_size_adjustment = step_size
                f_size = f_size + f_size_adjustment
                if (f_size<2):
                    f_size = 2
                    governor += 1
                    if governor > 2: loop = len(joining_text_list)
                if (f_size>3000):
                    f_size = 3000
                    governor += 1
                    if governor > 2: loop = len(joining_text_list)
                if use_text_size_modifier: # check to see if font size modifier is being used to begin changing size
                    if attack == loop:
                        decay = len(text) - attack
                        if even: 
                            decay -=2 
                        else:
                            decay -= 0
							
                        if decay < 1: decay = 1
                        size_difference,new_step_size = 0,0
						
                        if medial_text_size > final_text_size: size_difference = medial_text_size - final_text_size
                        if final_text_size > medial_text_size: size_difference = final_text_size - medial_text_size
						
                        if len(text)>3:
                            if decay>1:
                                if even:
                                    new_step_size = size_difference/decay
                                else:
                                    new_step_size = size_difference/(decay-1)
                            else:
                                new_step_size = size_difference/decay
                        else:
                            if decay>1:
                                new_step_size = size_difference/(decay-1)
                            else:
                                new_step_size = size_difference/decay
							
                        if not even: 
                            if final_text_size > medial_text_size:
						        f_size = medial_text_size + new_step_size
                                
                            else:
                                f_size -= (step_size + (new_step_size * 1))
						   
                        if even: f_size -= (step_size)
						
                        if (f_size<2): f_size = 2
                        if (f_size>3000): f_size = 3000
                        if final_text_size < medial_text_size: new_step_size = 0 - new_step_size
                        step_size = new_step_size
                kern_gap = (f_size * kern_factor)
        
		
		#### New text added to take account of adjusted font size for next character
        if use_text_size_modifier:
            if not even: 
                if final_text_size > medial_text_size:
                    temp_f_size = f_size + new_step_size
                else:
                    temp_f_size = f_size - new_step_size
						   
            else:
                if final_text_size > medial_text_size:
                    temp_f_size = f_size + new_step_size
                else:
                    temp_f_size = f_size - new_step_size
				
            if (temp_f_size<2): temp_f_size = 2
            if (temp_f_size>3000): temp_f_size = 3000
            (tempbox_width, tempbox_height, tempbox_ascent, tempbox_descent) = pdb.gimp_text_get_extents_fontname(NextChar, temp_f_size, PIXELS, str(f_name))
                
        else:
            (tempbox_width, tempbox_height, tempbox_ascent, tempbox_descent) = pdb.gimp_text_get_extents_fontname(NextChar, f_size, PIXELS, str(f_name))
								
        NextChr_width = tempbox_width
        loop = loop + 1
    if (keep_layers == 0):
        final_layer = pdb.gimp_image_get_active_layer(image)
        label_repeat_info = ""
        if repeat_text: label_repeat_info = " repeat"
        label_colour_info = ""
        if spread_colour: label_colour_info = " shaded"
        if use_gradient_colour: label_colour_info = " gradient"
        if initial_step_size <> 0: label_fontsize_info += " with " + str(initial_step_size) + "pt steps"
        if use_text_size_modifier:
            label_fontsize_info = " " + str(int(initial_text_size)) + "pt" + " to " + str(int(medial_text_size)) + "pt" + " to "  + str(int(final_text_size)) + "pt"
        label_details += label_repeat_info
        label_details += label_colour_info
        label_details += label_fontsize_info
        if use_space_to_change_step_size:
            size_method = " by word"
        else:
            size_method = " by char"
        label_details += size_method
        pdb.gimp_item_set_name(final_layer,label_details)
	
##########################################################################################################

#### End of proc_text_along_path
 	
#####################################################################################################

def fn_adjust_xpos(NextChar, CurrentChar, RealCurrentChar, RealNextChar, font_name_pointer, f_size, jtl):
    xpos_adjust = 0
	
    # adjust horizontal positioning for letter f 
    if (NextChar == unichr(102)):
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(f_alone_left_bearing[font_name_pointer], f_size)

    if (jtl == unichr(102)): # standalone letter f right bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(f_alone_right_bearing[font_name_pointer], f_size)
			
    if ((NextChar > unichr(57381)) and (NextChar < unichr(57387))): #  letter f and alternative f left bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(f_left_bearing[font_name_pointer], f_size)
			
    if (jtl > unichr(57381) and jtl < unichr(57387)): #  letter f and alternative f right bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(f_right_bearing[font_name_pointer], f_size)
			
    # adjust horizontal positioning for alternative letter f

    if (NextChar == unichr(57502)): # standalone letter f and standalone alternative f left bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(alt_f_alone_left_bearing[font_name_pointer], f_size)

    if (jtl == unichr(57502)): # standalone letter f and standalone alternative f right bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(alt_f_alone_right_bearing[font_name_pointer], f_size)

    if (NextChar > unichr(57502) and NextChar < unichr(57508)): #  letter f and alternative f left bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(alt_f_left_bearing[font_name_pointer], f_size)

    if (jtl > unichr(57502) and jtl < unichr(57508)): #  letter f and alternative f right bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(alt_f_right_bearing[font_name_pointer], f_size)

    # adjust horizontal positioning for letter j
		
    if (NextChar == unichr(106)): # standalone letter j left bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(j_alone_left_bearing[font_name_pointer], f_size)
				
    if (jtl == unichr(106)): # standalone letter j right bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment((j_alone_right_bearing[font_name_pointer]), f_size)
			
    if ((NextChar > unichr(57403) and NextChar < unichr(57409))): # join to letter j left bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(j_left_bearing[font_name_pointer], f_size)
				
    if ((jtl > unichr(57403) and jtl < unichr(57409)) ): # letter j right bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(j_right_bearing[font_name_pointer], f_size)

    # adjust horizontal positioning for alternative letter j
		
    if (NextChar == unichr(57516)): # standalone alternative letter j left bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(alt_j_alone_left_bearing[font_name_pointer], f_size)
				
    if (jtl == unichr(57516)): # standalone alternative letter j right bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment((alt_j_alone_right_bearing[font_name_pointer]), f_size)
			
    if (NextChar > unichr(57516) and NextChar < unichr(57523)): # join to alternative letter j left bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(alt_j_left_bearing[font_name_pointer], f_size)
				
    if (jtl > unichr(57516) and jtl < unichr(57523) ): # alternative letter j right bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(alt_j_right_bearing[font_name_pointer], f_size)

    # adjust horizontal positioning for letter p
		
    if (NextChar == unichr(112)): # standalone letter p left bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(p_alone_left_bearing[font_name_pointer], f_size)
				
    if (jtl == unichr(112)): # standalone letter p right bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(p_alone_right_bearing[font_name_pointer], f_size)
			
    if (NextChar > unichr(57437) and NextChar < unichr(57444)): # join to letter p left bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(p_left_bearing[font_name_pointer], f_size)
				
    if (jtl > unichr(57437) and jtl < unichr(57444)): # letter p right bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(p_right_bearing[font_name_pointer], f_size)

    # adjust horizontal positioning for letter t

    if (NextChar == unichr(116)): # standalone letter t left bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(t_alone_left_bearing[font_name_pointer], f_size)
				
    if (jtl == unichr(116)): # standalone letter t right bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(t_alone_right_bearing[font_name_pointer], f_size)
			
    if (NextChar > unichr(57460) and NextChar < unichr(57467)): # join to letter t left bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(t_left_bearing[font_name_pointer], f_size)
				
    if (jtl > unichr(57460) and jtl < unichr(57467)): # letter t right bearing adjustment
        if (RealCurrentChar <> RealNextChar):
            xpos_adjust += fn_font_x_pos_adjustment(t_right_bearing[font_name_pointer], f_size)
			
    # Kerning adjustments for Capital Letters
				
    if CurrentChar == "A": # adjust kerning after A where needed
        if (NextChar in "ABDEFHKLMNPRS"):
            xpos_adjust += fn_font_x_pos_adjustment(A_to_ABDEFHKLMNPRS_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(A_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(A_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(A_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in "IU"):
            xpos_adjust += fn_font_x_pos_adjustment(A_to_IU_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VW"):
            xpos_adjust += fn_font_x_pos_adjustment(A_to_VW_right_bearing[font_name_pointer], f_size)
        if (NextChar == "Y"):
            xpos_adjust += fn_font_x_pos_adjustment(A_to_Y_right_bearing[font_name_pointer], f_size)
        if (NextChar == "Z"):
            xpos_adjust += fn_font_x_pos_adjustment(A_to_Z_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(A_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2 or NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(A_to_lc2_3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(A_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(A_to_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6):
            xpos_adjust += fn_font_x_pos_adjustment(A_to_lc6_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(A_to_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(A_to_lc8_right_bearing[font_name_pointer], f_size)

    if CurrentChar == "B": # adjust kerning after B where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(B_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(B_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFLRS"):
            xpos_adjust += fn_font_x_pos_adjustment(B_to_BDEFLRS_right_bearing[font_name_pointer], f_size)
        if (NextChar == "I"):
            xpos_adjust += fn_font_x_pos_adjustment(B_to_I_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(B_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VW"):
            xpos_adjust += fn_font_x_pos_adjustment(B_to_VW_right_bearing[font_name_pointer], f_size)
        if (NextChar in "XYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(B_to_XYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(B_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(B_to_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6 or NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(B_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar  in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(B_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "C": # adjust kerning after C where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFLR"):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_BDEFLR_right_bearing[font_name_pointer], f_size)
        if (NextChar in "IKMNPU"):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_IKMNPU_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQS"):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_CGOQS_right_bearing[font_name_pointer], f_size)
        if (NextChar in "JT"):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_JT_right_bearing[font_name_pointer], f_size)
        if (NextChar == "M"):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_M_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VW"):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_VW_right_bearing[font_name_pointer], f_size)
        if (NextChar in "XYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_XYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_lc2_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_lc6_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(C_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "D": # adjust kerning after D where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(D_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFLR"):
            xpos_adjust += fn_font_x_pos_adjustment(D_to_BDEFLR_right_bearing[font_name_pointer], f_size)
        if (NextChar in "HKNPUCGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(D_to_HKNPUCGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "I"):
            xpos_adjust += fn_font_x_pos_adjustment(D_to_I_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(D_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "M"):
            xpos_adjust += fn_font_x_pos_adjustment(D_to_M_right_bearing[font_name_pointer], f_size)
        if (NextChar == "S"):
            xpos_adjust += fn_font_x_pos_adjustment(D_to_S_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(D_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in "XYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(D_to_XYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(D_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(D_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "E": # adjust kerning after E where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(E_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQS"):
            xpos_adjust += fn_font_x_pos_adjustment(E_to_CGOQS_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VWMXY"):
            xpos_adjust += fn_font_x_pos_adjustment(E_to_VWMXY_right_bearing[font_name_pointer], f_size)
        if (NextChar in "JT"):
            xpos_adjust += fn_font_x_pos_adjustment(E_to_JT_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(E_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6):
            xpos_adjust += fn_font_x_pos_adjustment(E_to_lc6_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(E_to_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(E_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "F": # adjust kerning after F where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar == "M"):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_M_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEHLNU"):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_BDEHLNU_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "FIKPRVW"):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_FIKPRVW_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "S"):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_S_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in "XYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_XYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_lc2_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_lc6_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(F_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "G": # adjust kerning after G where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(G_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFHIKLMNPRUS"):
            xpos_adjust += fn_font_x_pos_adjustment(G_to_BDEFHIKLMNPRUS_right_bearing[font_name_pointer], f_size)
        if (NextChar == "M"):
            xpos_adjust += fn_font_x_pos_adjustment(G_to_M_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(G_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(G_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in "XYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(G_to_XYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VW"):
            xpos_adjust += fn_font_x_pos_adjustment(G_to_VW_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(G_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2 or NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(G_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4 or NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(G_to_lc4_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6):
            xpos_adjust += fn_font_x_pos_adjustment(G_to_lc6_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(G_to_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(G_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "H": # adjust kerning after H where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(H_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(H_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "M"):
            xpos_adjust += fn_font_x_pos_adjustment(H_to_M_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(H_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VWXYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(H_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFHIKLNPRU"):
            xpos_adjust += fn_font_x_pos_adjustment(H_to_BDEFHIKLNPRU_right_bearing[font_name_pointer], f_size)
        if (NextChar in "COGQ"):
            xpos_adjust += fn_font_x_pos_adjustment(H_to_COGQ_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(H_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2 or NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(H_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4 or NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(H_to_lc4_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6 or NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(H_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(H_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "I": # adjust kerning after I where needed
        if (NextChar in "AM"):
            xpos_adjust += fn_font_x_pos_adjustment(I_to_AM_right_bearing[font_name_pointer], f_size)
        if (NextChar == "B"):
            xpos_adjust += fn_font_x_pos_adjustment(I_to_B_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(I_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "DEFHIKLNPRUS"):
            xpos_adjust += fn_font_x_pos_adjustment(I_to_DEFHIKLNPRUS_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(I_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(I_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VW"):
            xpos_adjust += fn_font_x_pos_adjustment(I_to_VW_right_bearing[font_name_pointer], f_size)
        if (NextChar in "XYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(I_to_XYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(I_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(I_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6):
            xpos_adjust += fn_font_x_pos_adjustment(I_to_lc6_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(I_to_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(I_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "J": # adjust kerning after J where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(J_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFHIKLMNPRTUXYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(J_to_BDEFHIKLMNPRTUXYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQS"):
            xpos_adjust += fn_font_x_pos_adjustment(J_to_CGOQS_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VW"):
            xpos_adjust += fn_font_x_pos_adjustment(J_to_VW_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(J_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2):
            xpos_adjust += fn_font_x_pos_adjustment(J_to_lc2_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(J_to_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4 or NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(J_to_lc4_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6 or NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(J_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(J_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "K": # adjust kerning after K where needed
        if (NextChar in "AUVWXY"):
            xpos_adjust += fn_font_x_pos_adjustment(K_to_AUVWXY_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFHIKLMNPRTZ"):
            xpos_adjust += fn_font_x_pos_adjustment(K_to_BDEFHIKLMNPRZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(K_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(K_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "S"):
            xpos_adjust += fn_font_x_pos_adjustment(K_to_S_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(K_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(K_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(K_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(K_to_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6):
            xpos_adjust += fn_font_x_pos_adjustment(K_to_lc6_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(K_to_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(K_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "L": # adjust kerning after L where needed
        if (NextChar in "AX"):
            xpos_adjust += fn_font_x_pos_adjustment(L_to_AX_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BEFR"):
            xpos_adjust += fn_font_x_pos_adjustment(L_to_BEFR_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(L_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "DHIKLMNPSUZ"):
            xpos_adjust += fn_font_x_pos_adjustment(L_to_DHIKLMNPSUZ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(L_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(L_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VW"):
            xpos_adjust += fn_font_x_pos_adjustment(L_to_VW_right_bearing[font_name_pointer], f_size)
        if (NextChar == "Y"):
            xpos_adjust += fn_font_x_pos_adjustment(L_to_Y_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(L_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2 or NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(L_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(L_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6):
            xpos_adjust += fn_font_x_pos_adjustment(L_to_lc6_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(L_to_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(L_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "M": # adjust kerning after M where needed
        if (NextChar in "AVWXY"):
            xpos_adjust += fn_font_x_pos_adjustment(M_to_AVWXY_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFHIKLMNPRSUZ"):
            xpos_adjust += fn_font_x_pos_adjustment(M_to_BDEFHIKLMNPRSUZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(M_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(M_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(M_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(M_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2 or NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(M_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(M_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6 or NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(M_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(M_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "N": # adjust kerning after N where needed
        if (NextChar in "AVWXY"):
            xpos_adjust += fn_font_x_pos_adjustment(N_to_AVWXY_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFHIKLMNPRTUZ"):
            xpos_adjust += fn_font_x_pos_adjustment(N_to_BDEFHIKLMNPRTUZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(N_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(N_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(N_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2 or NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(N_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6 or NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(N_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(N_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "O": # adjust kerning after O where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(O_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFHIKLNPRU"):
            xpos_adjust += fn_font_x_pos_adjustment(O_to_BDEFHKLNPRU_right_bearing[font_name_pointer], f_size)
        if (NextChar == "I"):
            xpos_adjust += fn_font_x_pos_adjustment(O_to_I_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(O_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "M"):
            xpos_adjust += fn_font_x_pos_adjustment(O_to_M_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VWXYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(O_to_VWXYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(O_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2 or NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(O_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(O_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(O_to_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(O_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "P": # adjust kerning after P where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFHKLNPRU"):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_BDEFHKLNPRU_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar == "I"):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_I_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "M"):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_M_right_bearing[font_name_pointer], f_size)
        if (NextChar == "S"):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_S_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VW"):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_VW_right_bearing[font_name_pointer], f_size)
        if (NextChar in "XYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_XYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_lc2_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_lc6_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(P_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "Q": # adjust kerning after Q where needed
        if (NextChar in "AJS"):
            xpos_adjust += fn_font_x_pos_adjustment(Q_to_AJS_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFHKLMNPRUZ"):
            xpos_adjust += fn_font_x_pos_adjustment(Q_to_BDEFHKLMNPRUZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(Q_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(Q_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VW"):
            xpos_adjust += fn_font_x_pos_adjustment(Q_to_VW_right_bearing[font_name_pointer], f_size)
        if (NextChar in "XY"):
            xpos_adjust += fn_font_x_pos_adjustment(Q_to_XY_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(Q_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6):
            xpos_adjust += fn_font_x_pos_adjustment(Q_to_lc6_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(Q_to_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(Q_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "R": # adjust kerning after R where needed
        if (NextChar in "ABDEFHIKLMNRPXZ"):
            xpos_adjust += fn_font_x_pos_adjustment(R_to_ABDEFHIKLMNRPXZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(R_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(R_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar in "SU"):
            xpos_adjust += fn_font_x_pos_adjustment(R_to_SU_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(R_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VWY"):
            xpos_adjust += fn_font_x_pos_adjustment(R_to_VWY_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(R_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(R_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6):
            xpos_adjust += fn_font_x_pos_adjustment(R_to_lc6_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(R_to_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(R_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "S": # adjust kerning after S where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFHIKLNPRU"):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_BDEFHIKLNPRU_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "I"):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_I_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "M"):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_M_right_bearing[font_name_pointer], f_size)
        if (NextChar == "S"):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_S_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VW"):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_VW_right_bearing[font_name_pointer], f_size)
        if (NextChar in "XY"):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_XY_right_bearing[font_name_pointer], f_size)
        if (NextChar == "Z"):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_Z_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_lc2_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6 or NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(S_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "T": # adjust kerning after T where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFLR"):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_BDEFLR_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "HKNPU"):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_HKNPU_right_bearing[font_name_pointer], f_size)
        if (NextChar == "I"):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_I_right_bearing[font_name_pointer], f_size)
        if (NextChar == "M"):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_M_right_bearing[font_name_pointer], f_size)
        if (NextChar == "S"):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_S_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VW"):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_VW_right_bearing[font_name_pointer], f_size)
        if (NextChar in "XYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_XYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_lc2_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6 or NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(T_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "U": # adjust kerning after U where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar == "B"):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_B_right_bearing[font_name_pointer], f_size)
        if (NextChar in "DEL"):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_DEL_right_bearing[font_name_pointer], f_size)
        if (NextChar in "HIKNPSUCGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_HIKNPSUCGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "F"):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_F_right_bearing[font_name_pointer], f_size)
        if (NextChar == "R"):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_R_right_bearing[font_name_pointer], f_size)
        if (NextChar in "XYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_XYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "M"):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_M_right_bearing[font_name_pointer], f_size)
        if (NextChar in "VW"):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_VW_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2 or NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4 or NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_lc4_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6 or NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(U_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "V": # adjust kerning after V where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(V_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFHIKLNPRSTUVWXYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(V_to_BDEFHIKLNPRSTUVWXYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(V_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(V_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "M"):
            xpos_adjust += fn_font_x_pos_adjustment(V_to_M_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(V_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(V_to_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4 or NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(V_to_lc4_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6 or NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(V_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(V_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "W": # adjust kerning after W where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(W_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFHIKLNPRSTUVWXYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(W_to_BDEFHIKLNPRSTUVWXYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(W_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(W_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "M"):
            xpos_adjust += fn_font_x_pos_adjustment(W_to_M_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(W_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(W_to_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(W_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(W_to_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6 or NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(W_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(W_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "X": # adjust kerning after X where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(X_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFHIKLNPRSTUVWXYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(X_to_BDEFHIKLNPRSTUVWXYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(X_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(X_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "M"):
            xpos_adjust += fn_font_x_pos_adjustment(X_to_M_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(X_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(X_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(X_to_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(X_to_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(X_to_lc8_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "Y": # adjust kerning after Y where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(Y_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFHIKLNPRTUVWXYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(Y_to_BDEFHIKLNPRTUVWXYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQ"):
            xpos_adjust += fn_font_x_pos_adjustment(Y_to_CGOQ_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(Y_to_J_right_bearing[font_name_pointer], f_size)
        if (NextChar == "M"):
            xpos_adjust += fn_font_x_pos_adjustment(Y_to_M_right_bearing[font_name_pointer], f_size)
        if (NextChar == "S"):
            xpos_adjust += fn_font_x_pos_adjustment(Y_to_S_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(Y_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1 or NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(Y_to_lc1_lc8_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2):
            xpos_adjust += fn_font_x_pos_adjustment(Y_to_lc2_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(Y_to_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(Y_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(Y_to_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6 or NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(Y_to_lc6_lc7_right_bearing[font_name_pointer], f_size)
				
    if CurrentChar == "Z": # adjust kerning after Z where needed
        if (NextChar == "A"):
            xpos_adjust += fn_font_x_pos_adjustment(Z_to_A_right_bearing[font_name_pointer], f_size)
        if (NextChar in "BDEFHIKLNPRUVWXYZ"):
            xpos_adjust += fn_font_x_pos_adjustment(Z_to_BDEFHIKLNPRUVWXYZ_right_bearing[font_name_pointer], f_size)
        if (NextChar in "CGOQS"):
            xpos_adjust += fn_font_x_pos_adjustment(Z_to_CGOQS_right_bearing[font_name_pointer], f_size)
        if (NextChar == "J"):
            xpos_adjust += fn_font_x_pos_adjustment(Z_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar == "M"):
            xpos_adjust += fn_font_x_pos_adjustment(Z_to_M_right_bearing[font_name_pointer], f_size)
        if (NextChar == "T"):
            xpos_adjust += fn_font_x_pos_adjustment(Z_to_T_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_1):
            xpos_adjust += fn_font_x_pos_adjustment(Z_to_lc1_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_2 or NextChar in lower_case_3):
            xpos_adjust += fn_font_x_pos_adjustment(Z_to_lc2_lc3_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_4):
            xpos_adjust += fn_font_x_pos_adjustment(Z_to_lc4_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_5):
            xpos_adjust += fn_font_x_pos_adjustment(Z_to_lc5_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_6):
            xpos_adjust += fn_font_x_pos_adjustment(Z_to_lc6_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_7):
            xpos_adjust += fn_font_x_pos_adjustment(Z_to_lc7_right_bearing[font_name_pointer], f_size)
        if (NextChar in lower_case_8):
            xpos_adjust += fn_font_x_pos_adjustment(Z_to_lc8_right_bearing[font_name_pointer], f_size)
			
    return xpos_adjust
	
#######################################################################################################
	
def fn_compensate_for_curve_in_path(p_angle, c_angle, f_size):
    curve_compensation,angle_diff = 0,0
	
    if (c_angle<0 and p_angle>180):
        c_angle += 360
    if (c_angle>180 and p_angle<0):
        c_angle -= 360
	
    if (c_angle>p_angle): # tests for text along path travelling in a clockwise direction
        
        min_angle = min(abs(p_angle), abs(c_angle))
        max_angle = max(abs(p_angle), abs(c_angle))
        angle_diff = (min_angle - max_angle)
        curve_compensation = fn_font_x_pos_adjustment(angle_diff, f_size)                

    if (c_angle<p_angle): # tests for  text along path travelling in a counter-clockwise direction
						
        min_angle = min(abs(p_angle), abs(c_angle))
        max_angle = max(abs(p_angle), abs(c_angle))
        angle_diff = max_angle - min_angle
        curve_compensation = fn_font_x_pos_adjustment(angle_diff,f_size)
		
    return curve_compensation,angle_diff

		
#######################################################################################################

def proc_show_joined_up_text_line(image, layer, text, f_name, f_colour, f_size, f_xpos, f_ypos, name_layer):
    layer_height = layer.height
    text_layer = pdb.gimp_text_layer_new(image, text, f_name, f_size, 0)
    pdb.gimp_image_insert_layer(image,text_layer,None,-1)
    pdb.gimp_text_layer_set_color(text_layer, f_colour)
    pdb.gimp_layer_translate(text_layer,f_xpos,image.height - (text_layer.height + f_ypos))
    pdb.gimp_item_set_name(text_layer, name_layer)

#######################################################################################################

def computeThetaWithSlope(fromPoint,toPoint,slope):
    dX=toPoint[0]-fromPoint[0]
    dY=toPoint[1]-fromPoint[1]

    if math.fabs(dX) < .001:
        theta=math.copysign(math.pi/2,dY)
    else:
        theta=math.atan(slope)
        if dX<0:
            theta=theta+math.pi
    return theta

#######################################################################################################

def fn_font_x_pos_adjustment(kerning, f_size):
    adjust_factor = (((100 * kerning) / 1024) * f_size) / 100
    return adjust_factor
	
#######################################################################################################

def fn_label_characters(label_text,label_len):
    label_char_info = label_text
    label_len_info = len(label_text)
    label_etc = ""
    if label_len_info > label_len: 
        label_char_info = label_char_info[0:label_len]
        if label_char_info.count(" ")>0:
            loop = 0
            space_pos = label_len - 1
            while loop<label_len:
                if label_char_info[loop] == " ": space_pos = loop
                loop += 1
            label_char_info = label_char_info[0:space_pos]
            label_etc = " etc"
    if label_len_info == 1: 
        label_len_detail = " char" 
    else:
        label_len_detail = " chars" 
    details = label_char_info + label_etc + " " + str(label_len_info ) + label_len_detail
    return details

#######################################################################################################

def fn_convert_text_to_joined_up_text(text, force_eflag, force_sflag, f_nohook_flag, g_loop_flag, j_loop_flag, k_unloop_flag, s_closed_flag, x_rounded_flag, y_loop_flag, z_loop_flag):

    # Start of conversion: text 

    TextLen = 0
    AllTextLen = 0
    Horizontal = ""
    Diagonal = ""
    NotJoined = ""
    PreviousChar = ""
    PreviousCharType = ""
    CurrentChar = ""
    CurrentCharType = ""
    NextChar = ""
    NextCharType = ""
    strTemp = ""
    ToRound = ""
    ToSlanted = ""
    ToStraight = ""
    ToSmall = ""
    ToTall = ""
	
    if (x_rounded_flag == True):
        HasHorizontal = "forvw" # define the letters which use horizontal joining strokes
        HasDiagonal = "abcdegjhijklmnpqstuxyz" # define the letters which use diagonal joining strokes
    else:
        HasHorizontal = "forvwx" # define the letters which use horizontal joining strokes
        HasDiagonal = "abcdegjhijklmnpqstuyz" # define the letters which use diagonal joining strokes

    HasNoJoin = " "  # define the lower case letters which have no joining strokes
    GoesToRound = "acdgoqs"
    GoesToSlanted = "vwxz"
    GoesToStraight = "bfhijklmnprtuy"
    GoesToSmall = "ijmnpruvwxyz"
    GoesToTall = "bfhklt"
    Horiz = "horizontal"
    Diagon = "diagonal"
    NoJoin = "notjoined"

    AllTextLen = len(text)
    TextLen = AllTextLen


    # Create a list to hold all the original characters of the text string
    # This list will remain unaltered
    loop = 0
    text_list = [text[loop]]
    loop = loop + 1
    while loop < len(text):
        if (text[loop] < " "):
            text_list.append(" ")
        else:
            text_list.append(text[loop])
        loop = loop + 1

    # Make a copy of the list to work with
    # This list will contain the individual characters of joined-up unicode text
    joining_text_list = text_list

    loop = 1
    NextChar = text_list[loop - 1]
    testing = ""
    NextCharType = NoJoin
    PreviousCharType = NoJoin
    CurrentCharType = NoJoin
    TestHasHorizontal = NextChar in HasHorizontal
    if (TestHasHorizontal == True):
        NextCharType = Horiz
        debug = NextCharType

    TestHasDiagonal = NextChar in HasDiagonal
    if (TestHasDiagonal == True):
        NextCharType = Diagon
    	
    if (TextLen>0):

        while (loop<=TextLen):
            testing = testing + joining_text_list[loop - 1]
            PreviousChar = CurrentChar
            PreviousCharType = CurrentCharType
            CurrentChar = NextChar
            CurrentCharType = NextCharType

            if (loop == TextLen):
                NextCharType = NoJoin
                NextChar = ""
            else:
                NextChar = text_list[loop]
                TestHasNoJoin = NextChar in HasNoJoin
                TestHasHorizontal = NextChar in HasHorizontal
                TestHasDiagonal = NextChar in HasDiagonal
                if (NextChar < "a" or NextChar > "z"):
                    NextCharType = NoJoin
                elif (TestHasNoJoin == True):
                    NextCharType = NoJoin
                elif (TestHasHorizontal == True):
                    NextCharType = Horiz
                else:
                    if (TestHasDiagonal == True):
                        NextCharType = Diagon
            # Deal with smart quote characters
            if (CurrentChar=="\"") and ((PreviousChar==" " or PreviousChar=="")):
                CurrentChar = unichr(8220)
            if (CurrentChar=="\'") and ((PreviousChar==" " or PreviousChar=="")):
                CurrentChar = unichr(8216)


            if ((CurrentChar < "a" or CurrentChar > "z") or loop == TextLen):
				
                if (loop == TextLen):

                # Force final character where alternative character is used
                    if (PreviousCharType == Diagon and CurrentChar == "e"):
                        CurrentChar = unichr(57368)
                        if (force_eflag == True):
                            if (PreviousCharType == NoJoin):
                                CurrentChar = unichr(57369)
                    
                    if (PreviousCharType == Horiz and CurrentChar == "e"):
                        CurrentChar = unichr(57369)
						
                    if (f_nohook_flag == True):
                        if (CurrentChar == "f"):
                            CurrentChar = unichr(57502)

                    if (g_loop_flag == True):
                        if (CurrentChar == "g"):
                            CurrentChar = unichr(57509)
							
                    if (j_loop_flag == True):
                        if (CurrentChar == "j"):
                            CurrentChar = unichr(57516)
							
                    if (k_unloop_flag == True):
                        if (CurrentChar == "k"):
                            CurrentChar = unichr(57523)
                    
                    if (CurrentChar == "o"):
                        CurrentChar = unichr(57435)
						
                    if (s_closed_flag == True):
                    # Rem Deal with previous horizontal join characters to closed s
                        if (PreviousChar == unichr(57382) or PreviousChar == unichr(57433) or PreviousChar == unichr(57450) or PreviousChar == unichr(57473) or PreviousChar == unichr(57478) or PreviousChar == unichr(57483) or PreviousChar == unichr(57503)): # Check for closed s joins from letters f, o, r, v, w
                            CurrentChar = unichr(57537)
                        else:                    
                            if (CurrentChar == "s"):
                                CurrentChar = unichr(57530)

                    if (x_rounded_flag == True):
                        if (CurrentChar == "x"):
                            CurrentChar = unichr(57544)

                    if (y_loop_flag == True):
                        if (CurrentChar == "y"):
                            CurrentChar = unichr(57551)

                    if (z_loop_flag == True):
                        if (CurrentChar == "z"):
                            CurrentChar = unichr(57558)									
							
					
                joining_text_list[loop - 1] = CurrentChar

			
            else:
                TestGoesToTall = NextChar in GoesToTall
                TestGoesToSmall = NextChar in GoesToSmall
                TestGoesToRound = NextChar in GoesToRound
                TestGoesToSlanted = NextChar in GoesToSlanted
                TestGoesToStraight = NextChar in GoesToStraight
                TestHasNoJoin = True #NextChar in HasNoJoin
                if (CurrentCharType == Diagon):
                    if (NextChar == "e"):
                        #Select e loop join for all diagonal characters
                        if (CurrentChar == "a"): 
                            CurrentChar = unichr(57344)
                        elif (CurrentChar == "b"): 
                            CurrentChar = unichr(57350)
                        elif (CurrentChar == "c"): 
                            CurrentChar = unichr(57356)
                        elif (CurrentChar == "d"): 
                            CurrentChar = unichr(57362)
                        elif (CurrentChar == "e"): 
                            CurrentChar = unichr(57370)
                            if (PreviousCharType == NoJoin): 
                                CurrentChar = unichr(57376)
                            else:
                                if (PreviousCharType == Horiz):
                                    CurrentChar = unichr(57376)
                        elif (CurrentChar == "g"): 
                            if (g_loop_flag == True):
                                CurrentChar = unichr(57510) # looped g to e
                            else:
                                CurrentChar = unichr(57387) # unlooped g to e
                        elif (CurrentChar == "h"): 
                            CurrentChar = unichr(57392)
                        elif (CurrentChar == "i"): 
                            CurrentChar = unichr(57398)
                        elif (CurrentChar == "j"): 
                            if (j_loop_flag == True):
                                CurrentChar = unichr(57517) # looped j to e
                            else:
                                CurrentChar = unichr(57404) # unlooped j to e
                        elif (CurrentChar == "k"):
                            if (k_unloop_flag == True):
                                CurrentChar = unichr(57524) # unlooped k to e
                            else:
                                CurrentChar = unichr(57409) # looped k to e
                        elif (CurrentChar == "l"): 
                            CurrentChar = unichr(57415)
                        elif (CurrentChar == "m"): 
                            CurrentChar = unichr(57421)
                        elif (CurrentChar == "n"): 
                            CurrentChar = unichr(57427)
                        elif (CurrentChar == "p"):
                            CurrentChar = unichr(57438)
                        elif (CurrentChar == "q"):
                            CurrentChar = unichr(57444)
                        elif (CurrentChar == "s"): 
                            if (s_closed_flag == True):
                                CurrentChar = unichr(57531) # closed s to e
                                # Rem Deal with previous horizontal join characters to closed s
                                if (PreviousChar == unichr(57382) or PreviousChar == unichr(57433) or PreviousChar == unichr(57450) or PreviousChar == unichr(57473) or PreviousChar == unichr(57478) or PreviousChar == unichr(57483) or PreviousChar == unichr(57503)): # Check for closed s joins from letters f, o, r, v, w
                                    CurrentChar = unichr(57538)
                            else:
                                CurrentChar = unichr(57455) # open s to e
                        elif (CurrentChar == "t"): 
                            CurrentChar = unichr(57461)
                        elif (CurrentChar == "u"):
                            CurrentChar = unichr(57467)
                        elif (CurrentChar == "x"):
                            if (x_rounded_flag == True):
                                CurrentChar = unichr(57545)
                        elif (CurrentChar == "y"):  
                            if (y_loop_flag == True):
                                CurrentChar = unichr(57552) # looped y to e
                            else:
                                CurrentChar = unichr(57488) # unlooped y to e
                        else:
                            if (CurrentChar == "z"):  
                                if (z_loop_flag == True):
                                    CurrentChar = unichr(57559) # looped z to e
                                else:
                                    CurrentChar = unichr(57493) # unlooped z to e

                    if (TestGoesToRound == True):
                        # Select rounded join for all diagonal characters
                        if (CurrentChar == "a"): 
                            CurrentChar = unichr(57345)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57344)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57348)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57348)
                        elif (CurrentChar == "b"): 
                            CurrentChar = unichr(57351)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57350)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57354)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57354)
                        elif (CurrentChar == "c"):
                            CurrentChar = unichr(57357)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57356)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57360)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57360)
                        elif (CurrentChar == "d"): 
                            CurrentChar = unichr(57363)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57362)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57366)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57366)
                        elif (CurrentChar == "e"): 
                            CurrentChar = unichr(57371)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57370)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57374)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57374)
                            if (PreviousCharType == NoJoin):
                                CurrentChar = unichr(57377)
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(57376)
                                    if (force_sflag == True):
                                        CurrentChar = unichr(57380)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(57380)
                            if (PreviousCharType == Horiz): 
                                CurrentChar = unichr(57377)
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(57376)
                                    #if (force_sflag == True):
                                        #CurrentChar = unichr(57377)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(57380)
                        elif (CurrentChar == "g"): 
                            if (g_loop_flag == True):
                                CurrentChar = unichr(57511) # looped g to round
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(57510)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(57514)
                            else:
                                CurrentChar = unichr(57388) # unlooped g to round
                        elif (CurrentChar == "j"): 
                            if (j_loop_flag == True):
                                CurrentChar = unichr(57518) # looped j to round
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(57517)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(57521)
                            else:
                                CurrentChar = unichr(57405) # unlooped j to round
                        elif (CurrentChar == "h"): 
                            CurrentChar = unichr(57393)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57392)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57396)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57396)
                                #if (force_sflag == True):
                                    #CurrentChar = unichr(57396)
                        elif (CurrentChar == "i"): 
                            CurrentChar = unichr(57399)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57398)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57402)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57402)
                        elif (CurrentChar == "k"): 
                            if (k_unloop_flag == True):
                                CurrentChar = unichr(57525) # unlooped k to round
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(57524)
                                    if (force_sflag == True):
                                        CurrentChar = unichr(57528)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(57528)
                            else:
                                CurrentChar = unichr(57410) # looped k to round
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(57409)
                                    if (force_sflag == True):
                                        CurrentChar = unichr(57413)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(57413)
                        elif (CurrentChar == "l"): 
                            CurrentChar = unichr(57416)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57415)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57419)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57419)
                        elif (CurrentChar == "m"):
                            CurrentChar = unichr(57422)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57421)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57425)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57425)
                        elif (CurrentChar == "n"): 
                            CurrentChar = unichr(57428)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57427)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57431)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57431)
                        elif (CurrentChar == "p"): 
                            CurrentChar = unichr(57439)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57438)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57442)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57442)
                        elif (CurrentChar == "q"):
                            CurrentChar = unichr(57445)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57444)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57448)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57448)
                        elif (CurrentChar == "s"): 
                            if (s_closed_flag == True):
                                CurrentChar = unichr(57532) # closed s to round
                                if (NextChar == "s"):
                                    CurrentChar = unichr(57535)
                                # Rem Deal with previous horizontal join characters to closed s
                                if (PreviousChar == unichr(57382) or PreviousChar == unichr(57433) or PreviousChar == unichr(57450) or PreviousChar == unichr(57473) or PreviousChar == unichr(57478) or PreviousChar == unichr(57483) or PreviousChar == unichr(57503)): # horizontal joins to closed s
                                    CurrentChar = unichr(57539)
                                    if (NextChar == "s"):
                                        CurrentChar = unichr(57542)
                            else:
                                CurrentChar = unichr(57456) # open s to round
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(57459)
                        elif (CurrentChar == "t"): 
                            CurrentChar = unichr(57462)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57461)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57465)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57465)
                        elif (CurrentChar == "u"): 
                            CurrentChar = unichr(57468)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57467)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57471)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57471)
                        elif (CurrentChar == "x"):
                            if (x_rounded_flag == True):
                                CurrentChar = unichr(57546)
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(57545)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(57549)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57549)
                        elif (CurrentChar == "y"): 
                            if (y_loop_flag == True):
                                CurrentChar = unichr(57553) # looped y to round
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(57552)
                                    if (force_sflag == True):
                                        CurrentChar = unichr(57556)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(57556)
                            else:
                                CurrentChar = unichr(57489) # unlooped y to round
                        else:
                            if (CurrentChar == "z"): 
                                if (z_loop_flag == True):
                                    CurrentChar = unichr(57560) # looped z to round
                                    if (NextChar == "s" and s_closed_flag == True):
                                        CurrentChar = unichr(57559)
                                    if (NextChar == "s" and s_closed_flag == False):
                                        CurrentChar = unichr(57563)
                                else:
                                    CurrentChar = unichr(57494) # unlooped z to round
                                    if (NextChar == "s" and s_closed_flag == True):
                                        CurrentChar = unichr(57493)
                                        if (force_sflag == True):
                                            CurrentChar = unichr(57497)
                                    if (NextChar == "s" and s_closed_flag == False):
                                        CurrentChar = unichr(57497)
                    if (TestGoesToSlanted == True):
                        # Select slanted join for all diagonal characters
                        if (CurrentChar == "a"): 
                            CurrentChar = unichr(57346)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(57349)
                        elif (CurrentChar == "b"): 
                            CurrentChar = unichr(57352)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(57355)
                        elif (CurrentChar == "c"): 
                            CurrentChar = unichr(57358)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(57361)
                        elif (CurrentChar == "d"): 
                            CurrentChar = unichr(57364)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(57367)
                        elif (CurrentChar == "e"): 
                            CurrentChar = unichr(57372)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(57375)
                            if (PreviousCharType == NoJoin):
                                CurrentChar = unichr(57378)
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(57381)
                            if (PreviousCharType == Horiz): 
                                CurrentChar = unichr(57378)
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(57381)
                        elif (CurrentChar == "g"): 
                            if (g_loop_flag == True):
                                CurrentChar = unichr(57512) # looped g to slanted
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(57515)
                            else:
                                CurrentChar = unichr(57389) # unlooped g to slanted
                        elif (CurrentChar == "h"): 
                            CurrentChar = unichr(57394)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(57397)
                        elif (CurrentChar == "i"): 
                            CurrentChar = unichr(57400)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(57403)
                        elif (CurrentChar == "j"): 
                            if (j_loop_flag == True):
                                CurrentChar = unichr(57519) # looped j to slanted
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(57522)
                            else:
                                CurrentChar = unichr(57406) # unlooped j to slanted
                        elif (CurrentChar == "k"): 
                            if (k_unloop_flag == True):
                                CurrentChar = unichr(57526) # unlooped k to slanted
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(57529)
                            else:
                                CurrentChar = unichr(57411) # looped k to slanted
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(57414)
                        elif (CurrentChar == "l"): 
                            CurrentChar = unichr(57417)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(57420)
                        elif (CurrentChar == "m"): 
                            CurrentChar = unichr(57423)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(57426)
                        elif (CurrentChar == "n"): 
                            CurrentChar = unichr(57429)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(57432)
                        elif (CurrentChar == "p"): 
                            CurrentChar = unichr(57440)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(57443)
                        elif (CurrentChar == "q"): 
                            CurrentChar = unichr(57446)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(57449)
                        elif (CurrentChar == "s"): 
                            if (s_closed_flag == True):
                                CurrentChar = unichr(57533) # closed s to slanted
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(57536)
                                # Rem Deal with previous horizontal join characters to closed s
                                if (PreviousChar == unichr(57382) or PreviousChar == unichr(57433) or PreviousChar == unichr(57450) or PreviousChar == unichr(57473) or PreviousChar == unichr(57478) or PreviousChar == unichr(57483) or PreviousChar == unichr(57503)): # slanted joins to closed s
                                    CurrentChar = unichr(57540)
                                    if (NextChar == "x" or NextChar == "z"):
                                        CurrentChar = unichr(57543)
                                if (NextChar == "s"):
                                    CurrentChar = unichr(57533)
                            else:
                                CurrentChar = unichr(57457) # open s to slanted
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(57460)
                        elif (CurrentChar == "t"): 
                            CurrentChar = unichr(57463)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(57466)
                        elif (CurrentChar == "u"): 
                            CurrentChar = unichr(57469)
                            if (NextChar == "x" or NextChar == "z"):
                                CurrentChar = unichr(57472)
                        elif (CurrentChar == "x"):
                            if (x_rounded_flag == True):
                                CurrentChar = unichr(57547)
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(57550)
                        elif (CurrentChar == "y"): 
                            if (y_loop_flag == True):
                                CurrentChar = unichr(57554) # looped y to slanted
                                if (NextChar == "x" or NextChar == "z"):
                                    CurrentChar = unichr(57557)
                            else:
                                CurrentChar = unichr(57490) # unlooped y to slanted
                        else:
                            if (CurrentChar == "z"): 
                                if (z_loop_flag == True):
                                    CurrentChar = unichr(57561) # looped z to slanted
                                    if (NextChar == "x" or NextChar == "z"):
                                        CurrentChar = unichr(57564)
                                else:
                                    CurrentChar = unichr(57495) # unlooped z to slanted
                                    if (NextChar == "x" or NextChar == "z"):
                                        CurrentChar = unichr(57498)
                    if (TestGoesToStraight == True):
                        # Select straight join for all diagonal characters
                        if (CurrentChar == "a"): 
                            CurrentChar = unichr(57347)
                        elif (CurrentChar == "b"): 
                            CurrentChar = unichr(57353)
                        elif (CurrentChar == "c"): 
                            CurrentChar = unichr(57359)
                        elif (CurrentChar == "d"): 
                            CurrentChar = unichr(57365)
                        elif (CurrentChar == "e"): 
                            CurrentChar = unichr(57373)
                            if (PreviousCharType == NoJoin): 
                                CurrentChar = unichr(57379)
                            if (PreviousCharType == Horiz): 
                                CurrentChar = unichr(57379)
                        elif (CurrentChar == "g"): 
                            if (g_loop_flag == True):
                                CurrentChar = unichr(57513) # looped g to straight
                            else:
                                CurrentChar = unichr(57390) # unlooped g to straight
                        elif (CurrentChar == "h"): 
                            CurrentChar = unichr(57395)
                        elif (CurrentChar == "i"): 
                            CurrentChar = unichr(57401)
                        elif (CurrentChar == "j"): 
                            if (j_loop_flag == True):
                                CurrentChar = unichr(57520) # looped j to straight
                            else:
                                CurrentChar = unichr(57407) # unlooped j to straight
                        elif (CurrentChar == "k"): 
                            if (k_unloop_flag == True):
                                CurrentChar = unichr(57527) # unlooped k to straight
                            else:
                                CurrentChar = unichr(57412) # looped k to straight
                        elif (CurrentChar == "l"): 
                            CurrentChar = unichr(57418)
                        elif (CurrentChar == "m"): 
                            CurrentChar = unichr(57424)
                        elif (CurrentChar == "n"): 
                            CurrentChar = unichr(57430)
                        elif (CurrentChar == "p"): 
                            CurrentChar = unichr(57441)
                        elif (CurrentChar == "q"):
                            CurrentChar = unichr(57447) #Use this join for q to u
                        elif (CurrentChar == "s"): 
                            if (s_closed_flag == True):
                                CurrentChar = unichr(57534) # closed s to straight
                                # Rem Deal with previous horizontal join characters to closed s
                                if (PreviousChar == unichr(57382) or PreviousChar == unichr(57433) or PreviousChar == unichr(57450) or PreviousChar == unichr(57473) or PreviousChar == unichr(57478) or PreviousChar == unichr(57483) or PreviousChar == unichr(57503)): # slanted joins to closed s
                                    CurrentChar = unichr(57541)
                                if (NextChar == "s"):
                                    CurrentChar = unichr(57534)
                            else:
                                CurrentChar = unichr(57458) # open s to straight
                        elif (CurrentChar == "t"): 
                            CurrentChar = unichr(57464)
                        elif (CurrentChar == "u"): 
                            CurrentChar = unichr(57470)
                        elif (CurrentChar == "x"):
                            if (x_rounded_flag == True):
                                CurrentChar = unichr(57548)
                        elif (CurrentChar == "y"): 
                            if (y_loop_flag == True):
                                CurrentChar = unichr(57555) # looped y to straight
                            else:
                                CurrentChar = unichr(57491) # unlooped y to straight
                        else:
                            if (CurrentChar == "z"): 
                                if (z_loop_flag == True):
                                    CurrentChar = unichr(57562) # looped z to straight
                                else:
                                    CurrentChar = unichr(57496) # unlooped z to straight

                     
                    if (TestHasNoJoin == True):
                        if (CurrentChar == "e"):
                            if (PreviousCharType == Diagon): 
                                CurrentChar = unichr(57368)
                            if (PreviousCharType == Horiz): 
                                CurrentChar = unichr(57369)
                
                if (CurrentCharType == Horiz):
                    if (NextChar == "e"):
                        # Select e loop join for all horizontal characters
                        if (CurrentChar == "f"): 
                            if (f_nohook_flag == True): # forcing hooked f to unhooked f
                                CurrentChar = unichr(57503) # unhooked f to e
                            else:
                                CurrentChar = unichr(57382) # hooked f to e
                        elif (CurrentChar == "o"): 
                            CurrentChar = unichr(57433)
                        elif (CurrentChar == "r"): 
                            CurrentChar = unichr(57450)
                        elif (CurrentChar == "v"): 
                            CurrentChar = unichr(57473)
                        elif (CurrentChar == "w"): 
                            CurrentChar = unichr(57478)
                        else:
                            if (CurrentChar == "x"):
                                if (x_rounded_flag == True): # forcing straight x to rounded x
                                    CurrentChar = unichr(57545) # rounded x to e
                                else:
                                    CurrentChar = unichr(57483) # straight x to e
                    elif (TestGoesToRound == True):
                        # Select rounded join for all horizontal characters
                        if (CurrentChar == "f"): 
                            if (f_nohook_flag == True): # forcing hooked f to unhooked f
                                CurrentChar = unichr(57504) # unhooked f to round
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(57503)
                                    if (force_sflag == True):
                                        CurrentChar = unichr(57507)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(57507)
                            else:
                                CurrentChar = unichr(57383) # hooked f to round
                                if (NextChar == "s" and s_closed_flag == True):
                                    CurrentChar = unichr(57382)
                                    if (force_sflag == True):
                                        CurrentChar = unichr(57386)
                                if (NextChar == "s" and s_closed_flag == False):
                                    CurrentChar = unichr(57386)
                        elif (CurrentChar == "o"): 
                            CurrentChar = unichr(57434)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57433)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57437)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57437)
                        elif (CurrentChar == "r"): 
                            CurrentChar = unichr(57451)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57450)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57454)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57454)
                        elif (CurrentChar == "v"): 
                            CurrentChar = unichr(57474)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57473)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57477)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57477)
                        elif (CurrentChar == "w"): 
                            CurrentChar = unichr(57479)
                            if (NextChar == "s" and s_closed_flag == True):
                                CurrentChar = unichr(57478)
                                if (force_sflag == True):
                                    CurrentChar = unichr(57482)
                            if (NextChar == "s" and s_closed_flag == False):
                                CurrentChar = unichr(57482)
                        else:
                            if (CurrentChar == "x"): 
                                if (x_rounded_flag == True): # forcing straight x to rounded x
                                    CurrentChar = unichr(57546) # rounded x to rounded
                                else:
                                    CurrentChar = unichr(57484) # straight x to rounded
                                    if (NextChar == "s" and s_closed_flag == True):
                                        CurrentChar = unichr(57483)
                                        if (force_sflag == True):
                                            CurrentChar = unichr(57487)
                                    if (NextChar == "s" and s_closed_flag == False):
                                        CurrentChar = unichr(57487)
                    elif (TestGoesToSmall == True):
                        if (CurrentChar == "f"): 
                            if (f_nohook_flag == True): # forcing hooked f to unhooked f
                                CurrentChar = unichr(57505) # unhooked f to slanted
                            else:
                                CurrentChar = unichr(57384) # hooked f to slanted
                        elif (CurrentChar == "o"): 
                            CurrentChar = unichr(57435)
                        elif (CurrentChar == "r"): 
                            CurrentChar = unichr(57452)
                        elif (CurrentChar == "v"): 
                            CurrentChar = unichr(57475)
                        elif (CurrentChar == "w"): 
                            CurrentChar = unichr(57480)
                        else:
                            if (CurrentChar == "x"): 
                                if (x_rounded_flag == True): # forcing straight x to rounded x
                                    CurrentChar = unichr(57547) # rounded x to small
                                else:
                                    CurrentChar = unichr(57485) # straight x to small
                    else:
                        if (TestGoesToTall == True):
                            if (CurrentChar == "f"): 
                                if (f_nohook_flag == True): # forcing hooked f to unhooked f
                                    CurrentChar = unichr(57506) # unhooked f to straight
                                else:
                                    CurrentChar = unichr(57385) # hooked f to straight
                            elif (CurrentChar == "o"): 
                                CurrentChar = unichr(57436)
                            elif (CurrentChar == "r"): 
                                CurrentChar = unichr(57453)
                            elif (CurrentChar == "v"): 
                                CurrentChar = unichr(57476)
                            elif (CurrentChar == "w"): 
                                CurrentChar = unichr(57481)
                            else:
                                if (CurrentChar == "x"):
                                    if (x_rounded_flag == True): # forcing straight x to rounded x
                                        CurrentChar = unichr(57548) # rounded x to tall
                                    else:
                                        CurrentChar = unichr(57486) # straight x to tall
                        if (TestHasNoJoin == True):
                            if (CurrentChar == "o"): 
                                CurrentChar = unichr(57435)

                if (force_eflag == True):
                    #Deal with letter e after an unjoined or joined descender
                    if (CurrentChar == unichr(57370)): # joined looped e to e
                        if (PreviousChar == unichr(57387)): # unjoined g to e
                            CurrentChar = unichr(57376) # unjoined e to e
                        if (PreviousChar == unichr(57404)): # unjoined j to e
                            CurrentChar = unichr(57376) # unjoined e to e
                        if (PreviousChar == unichr(57488)): # unjoined y to e
                            CurrentChar = unichr(57376) # unjoined e to e
                    if (CurrentChar == unichr(57371)): # joined looped e to round letter
                        if (PreviousChar == unichr(57387)): # unjoined g to e
                            CurrentChar = unichr(57377) # unjoined e to round letter
                        if (PreviousChar == unichr(57404)): # unjoined j to e
                            CurrentChar = unichr(57377) # unjoined e to round letter
                        if (PreviousChar == unichr(57488)): # unjoined y to e
                            CurrentChar = unichr(57377) # unjoined e to round letter
                    if (CurrentChar == unichr(57372)): # joined looped e to slanted letter
                        if (PreviousChar == unichr(57387)): # unjoined g to e
                            CurrentChar = unichr(57378) # unjoined e to slanted
                        if (PreviousChar == unichr(57404)): # unjoined j to e
                            CurrentChar = unichr(57378) # unjoined e to slanted
                        if (PreviousChar == unichr(57488)): # unjoined y to e
                            CurrentChar = unichr(57378) # unjoined e to slanted
                    if (CurrentChar == unichr(57373)): # joined looped e to straight
                        if (PreviousChar == unichr(57387)): # unjoined g to e
                            CurrentChar = unichr(57379) # unjoined e to straight
                        if (PreviousChar == unichr(57404)): # unjoined j to e
                            CurrentChar = unichr(57379) # unjoined e to straight
                        if (PreviousChar == unichr(57488)): # unjoined y to e
                            CurrentChar = unichr(57379) # unjoined e to straight
                    if (CurrentChar == unichr(57368)): # looped e unjoined
                        if ((PreviousChar == unichr(57387)) or (PreviousChar == unichr(57388)) or (PreviousChar == unichr(57389))  or (PreviousChar == unichr(57390)) ): # all unjoined g letters
                            CurrentChar = unichr(57369) # unlooped e unjoined
                        if ((PreviousChar == unichr(57404)) or (PreviousChar == unichr(57405)) or (PreviousChar == unichr(57406))  or (PreviousChar == unichr(57407) )): # all unjoined j letters
                            CurrentChar = unichr(57369) # unlooped e unjoined
                        if ((PreviousChar == unichr(57488)) or (PreviousChar == unichr(57489)) or (PreviousChar == unichr(57490))  or (PreviousChar == unichr(57491)) ): # all unjoined y letters
                            CurrentChar = unichr(57369) # unlooped e unjoined

                if (f_nohook_flag == True):
                    if (CurrentChar == "f"):
                        CurrentChar = unichr(57502) # force ordinary hooked f to unhooked f
                if (g_loop_flag == True):
                    if (CurrentChar == "g"):
                        CurrentChar = unichr(57509) # force ordinary unlooped g to looped g
                if (j_loop_flag == True):
                    if (CurrentChar == "j"):
                        CurrentChar = unichr(57516) # force ordinary unlooped j to looped j
                if (k_unloop_flag == True):
                    if (CurrentChar == "k"):
                        CurrentChar = unichr(57523) # force ordinary looped k to unlooped k
                if (s_closed_flag == True):
                    if (CurrentChar == "s"):
                        CurrentChar = unichr(57530) # force ordinary open s to closed s
                        if (PreviousChar == unichr(57382) or PreviousChar == unichr(57433) or PreviousChar == unichr(57450) or PreviousChar == unichr(57473) or PreviousChar == unichr(57478) or PreviousChar == unichr(57483) or PreviousChar == unichr(57503)): # Check for closed s joins from letters f, o, r, v, w
                            CurrentChar = unichr(57537)
                if (x_rounded_flag == True):
                    if (CurrentChar == "x"):
                        CurrentChar = unichr(57544) # force ordinary straight x to rounded x
                if (y_loop_flag == True):
                    if (CurrentChar == "y"):
                        CurrentChar = unichr(57551) # force ordinary unlooped y to looped y
                if (z_loop_flag == True):
                    if (CurrentChar == "z"):
                        CurrentChar = unichr(57558) # force ordinary unlooped z to looped z
						
            joining_text_list[loop - 1] = CurrentChar
			
            loop = loop + 1
		

    loop = 0	
    joined_text = ""
    while (loop < len(joining_text_list)):
        joined_text = joined_text + joining_text_list[loop]
        loop = loop + 1
    label_text = ""
    loop = 0
    while (loop < len (text)):
        if (text[loop] >= " "):
            label_text += text[loop]

        loop += 1
	
    return joined_text, label_text

#######################################################################################################

# Plugin Registration and User Interface Settings

#######################################################################################################




register(
    "python_fu_JoinedUp_Text_v2-17",
    "To emulate joined up writing ",
    "To emulate joined up writing ",
    "KGH",
    "Open source (BSD 3-clause license)",
    "2018",
    "<Image>/Filters/Text FX/Joined-up Text v2.17...",
    "*",
    [
    (PF_FONT, "pf_font", "Font", "ItalicVariations Italic"),
    (PF_SPINNER, "pf_fontsize", "Font size:", 200, (1, 3000, 1)),
    (PF_COLOR, "pf_color", "Text colour:", (0.0, 0.0, 0.0)),
    (PF_COLOR, "pf_color2", "Colour to shade to:", (255, 255, 255)),
    (PF_TOGGLE, "pf_shade_text", "Shade colour along text:", 0),
    (PF_GRADIENT, "pf_current_gradient", "Gradient for text colours:", 0),
    (PF_TOGGLE, "pf_gradient_text", "Colour text from\ncurrent gradient:", 0),
    (PF_FILENAME, "pf_xcffilename", ".xcf font file:", 0),
    (PF_TOGGLE, "pf_usecustomfont", "Use custom font:", 0),
    (PF_STRING, "pf_string", "Text:", "Let the fun begin"),
    #(PF_TEXT, "text", "Multiline Text:", "Let the fun begin"),
    (PF_SLIDER, "pf_opacity", "Letter Layer Opacity:", 100,(0,100,1)),
    (PF_TOGGLE, "pf_reversed_writing", "As Reversed Writing:", 0),
    (PF_TOGGLE, "pf_repeat_text", "Repeat text along path:", 0),
    (PF_SPINNER, "pf_text_size_factor", "Text size step change:", -2,(-100,100,0.5)),
    (PF_TOGGLE, "pf_use_text_size_modifier", "Use text size modifiers:", 0),
    (PF_SPINNER, "pf_initial_text_size", "Initial text size:", 100,(2,3000,1)),
    (PF_SPINNER, "pf_medial_text_size", "Target text size:", 200,(2,3000,1)),
    (PF_SPINNER, "pf_final_text_size", "Final text size:", 100,(2,3000,1)),
    (PF_SLIDER, "pf_position_of_target", "Target for change in text\nsize as percentage of path:", 50,(1,99,5)),
    (PF_TOGGLE, "pf_space_alters_size", "Change size with spaces:", 0),
    (PF_SPINNER, "pf_compensate_multiplier", "Try to compensate for\nchanges in path direction:", 0,(0,10,0.5)),
    (PF_SPINNER, "pf_kerning_factor", "Adjust spacing \nbetween characters:", 0,(-500,500,1)),
    (PF_TOGGLE, "pf_rotate_text", "Rotate letters:", 0),
    (PF_SLIDER, "pf_user_angle", "Angle of rotation:", 0,(-180,180,1)),
    (PF_TOGGLE, "pf_rotate_jitter", "Use jitter:", 0),
    (PF_SPINNER, "pf_jitter_range", "Jitter range:", 0,(0,359,1)),
    (PF_TOGGLE, "pf_retain_layers", "Retain layers:", 0),
    (PF_TOGGLE, "pf_retain_paths", "Retain paths:", 0),
    (PF_TOGGLE, "pf_force_unhooked_f", "Alternative joined-up letter f:", 0),
    (PF_TOGGLE, "pf_force_looped_g", "Alternative joined-up letter g:", 0),
    (PF_TOGGLE, "pf_force_looped_j", "Alternative joined-up letter j:", 0),
    (PF_TOGGLE, "pf_force_unlooped_k", "Alternative joined-up letter k:", 0),
    (PF_TOGGLE, "pf_force_closed_s", "Alternative joined-up letter s:", 0),
    (PF_TOGGLE, "pf_force_rounded_x", "Alternative joined-up letter x:", 0),
    (PF_TOGGLE, "pf_force_looped_y", "Alternative joined-up letter y:", 0),
    (PF_TOGGLE, "pf_force_looped_z", "Alternative joined-up letter z:", 0),
    ],
    [],
    proc_main_plugin)

main()

